<?php
function imagepress_count_user_posts_by_type( $userid ) {
    global $wpdb;

    $imagepress_slug = imagepress_get_option( 'ip_slug' );

    $where = get_posts_by_author_sql( $imagepress_slug, true, $userid );
    $count = $wpdb->get_var( "SELECT COUNT(*) FROM $wpdb->posts $where" );

    return apply_filters( 'get_usernumposts', $count, $userid );
}

function imagepress_post_views($userId, $count = true) {
    $axCount = get_user_meta($userId, 'ax_post_views', true);
    if ($axCount == '') {
        $axCount = 0;
    }

    if ($count === true) {
        $axCount++;
        update_user_meta($userId, 'ax_post_views', $axCount);
    }

    return $axCount;
}

function imagepress_author_base() {
    global $wp_rewrite;

    $cinnamon_author_slug = imagepress_get_option( 'cinnamon_author_slug' );

    $wp_rewrite->author_base      = $cinnamon_author_slug;
    $wp_rewrite->author_structure = '/' . $wp_rewrite->author_base . '/%author%';
}

function imagepress_get_related_author_posts( $author ) {
    $imagepress_slug = imagepress_get_option('ip_slug');
    $authors_posts = get_posts([
        'author' => $author,
        'posts_per_page' => 9,
        'post_type' => $imagepress_slug
    ]);

    $output = '';

    if ($authors_posts) {
        $output .= '<div class="cinnamon-grid">
            <ul>';
                foreach ($authors_posts as $authors_post) {
                    $output .= '<li><a href="' . get_permalink($authors_post->ID) . '">' . get_the_post_thumbnail($authors_post->ID, 'thumbnail') . '</a></li>';
                }
            $output .= '</ul>
        </div>';
    }

    return $output;
}

function imagepress_extra_contact_info( $contactmethods ) {
    unset( $contactmethods['aim'] );
    unset( $contactmethods['yim'] );
    unset( $contactmethods['jabber'] );
    unset( $contactmethods['googleplus'] );

    $contactmethods['twitter']  = 'X';
    $contactmethods['instagram']  = 'Instagram';
    $contactmethods['linkedin']  = 'LinkedIn';
    $contactmethods['facebook'] = 'Facebook';
    $contactmethods['pinterest']  = 'Pinterest';

    return $contactmethods;
}



function imagepress_get_users_by_post_count( $post_type = 'post' ) {
    global $wpdb;

    $users = $wpdb->get_results(
        $wpdb->prepare(
            "SELECT {$wpdb->users}.ID, p.post_count
            FROM {$wpdb->users}  
            LEFT JOIN (
                SELECT post_author, COUNT(*) AS post_count 
                FROM {$wpdb->posts} WHERE post_type = %s
                GROUP BY post_author
            ) p ON {$wpdb->users}.id = p.post_author 
            ORDER BY p.post_count DESC",
            $post_type
        )
    );

    return $users;
}



function imagepress_canonical_url( $canonical_url, $post ) {
    if ( is_page( (int) imagepress_get_option( 'ip_profile_page' ) ) ) {
        if ( isset( $_GET['author'] ) && $_GET['author'] !== '' ) {
            $canonical_url = trailingslashit( get_permalink() ) . '?' . imagepress_get_option( 'cinnamon_author_slug' ) . '=' . $_GET['author'];
        }
    }

    return $canonical_url;
}
add_filter( 'get_canonical_url', 'imagepress_canonical_url', 10, 2 );



/* CINNAMON CARD SHORTCODE */
function imagepress_card( $atts ) {
    extract(shortcode_atts([
        'author' => '',
        'sort' => 0,
        'role' => ''
    ], $atts));

    $imagepress_slug = imagepress_get_option( 'ip_slug' );
    $ip_image_size   = imagepress_get_option( 'ip_image_size' );

    $display = '<div id="author-cards">';

    // Filter users by role
    $role = sanitize_text_field( $role );
    $role = ( (string) $role === '' ) ? '' : '&role=' . $role;

    // Get all users with at least one image uploaded
    // $number = $count;
    // $paged  = ( get_query_var( 'paged' ) ) ? get_query_var( 'paged' ) : 1;
    // $offset = ( $paged - 1 ) * $number;
    // $users  = get_users( $role );
    //$query = get_users( '&number=-1' . $role );
    $query = imagepress_get_users_by_post_count( $imagepress_slug );

    /*
    $total_users = $users ? count( $users ) : 0;
    $total_query = $query ? count( $query ) : 0;
    $total_pages = intval( $total_users / $number ) + 1;
    /**/

    $display .= '<ul class="list" id="cinnamon-cards--paginated" data-app=' . imagepress_get_option( 'ip_app' ) . '>';

    /*
    $args = array(
    'role' => '', // Specify the desired user role
    'orderby' => 'post_count',
    'order' => 'DESC',
    'number' => -1 // Retrieve all users
);

$user_query = new WP_User_Query($args);
$users = $user_query->get_results();
/**/
    //

    foreach ( $query as $q ) {
        $author = $q->ID;

        $card_args = [
            'author'         => $author,
            'posts_per_page' => imagepress_get_option( 'ip_cards_per_author' ),
            'post_type'      => $imagepress_slug,
        ];

        $authors_posts = get_posts( $card_args );

        if ( count( $authors_posts ) > 0 ) {
            $display .= '<li class="cinnamon-card" style="width:' . get_option( 'cinnamon_card_width', 100 ) . '%">
                <div class="cinnamon-card--inner">';
                    $hub_user_info = get_userdata( $author );
                    $hub_user      = $hub_user_info->display_name;

                    if ( ! empty( $hub_user_info->first_name ) ) {
                        $hub_user = $hub_user_info->first_name . ' ' . $hub_user_info->last_name;
                    }

                    $display .= '<div class="avatar-holder">
                        <a href="' . imagepress_get_profile_uri( $author, false ) . '">' . get_avatar( $author, 90 ) . '</a>
                        <h3><a href="' . imagepress_get_profile_uri( $author, false ) . '" class="name">' . $hub_user . '</a></h3>
                    </div>';

                    $display .= '<div class="cinnamon-stats">
                        <div class="cinnamon-meta"><b>' . imagepress_kformat( imagepress_post_views( $author, false ) ) . '</b><br><small class="ui--d">' . __( 'views', 'imagepress' ) . '</small></div>
                        <div class="cinnamon-meta"><b>' . imagepress_kformat( imagepress_get_follower_count( $author ) ) . '</b><br><small class="ui--d">' . __( 'followers', 'imagepress' ) . '</small></div>
                        <div class="cinnamon-meta"><b>' . imagepress_kformat( imagepress_count_user_posts_by_type( $author, $imagepress_slug ) ) . '</b><br><small class="ui--d">' . __( 'uploads', 'imagepress' ) . '</small></div>
                    </div>
                </div>';

                if ( $authors_posts ) {
                    $display .= '<div class="mosaicflow imagepress--equal-height">';
                        foreach ( $authors_posts as $authors_post ) {
                            $display .= '<a href="' . get_permalink( $authors_post->ID ) . '">' . get_the_post_thumbnail( $authors_post->ID, $ip_image_size ) . '</a>';
                        }
                    $display .= '</div>';
                }
            $display .= '</li>';
        }
    }

    $display .= '</ul>

    <div id="pagination-buttons"></div>';

    /*
    if ( (int) $total_users > (int) $total_query ) {
        $display .= '<div id="pagination" class="native-pagination">';
        $display .= '<span class="pages">' . __( 'Pages:', 'imagepress' ) . '</span>';
        $current_page = max( 1, get_query_var( 'paged' ) );
        $display .= paginate_links(
            [
                'base'      => get_pagenum_link( 1 ) . '%_%',
                'format'    => 'page/%#%/',
                'current'   => $current_page,
                'total'     => $total_pages,
                'prev_next' => false,
                'type'      => 'list',
            ]
        );
        $display .= '</div>';
    }
    /**/

    $display .= '</div>';

    return $display;
}



/* CINNAMON PROFILE SHORTCODE */
function imagepress_profile( $atts ) {
    $atts = shortcode_atts(
        [
            'author'   => '',
            'username' => '',
        ],
        $atts
    );

    $author   = $atts['author'];
    $username = $atts['username'];

    global $wpdb, $current_user;

    $cinnamon_author_slug = (string) imagepress_get_option( 'cinnamon_author_slug' );

    // If no user is provided, get logged in user
    if ( ! isset( $_GET[ $cinnamon_author_slug ] ) ) {
        $current_user = wp_get_current_user();

        if ( ! ( $current_user instanceof WP_User ) ) {
            return;
        } else {
            $_GET[ $cinnamon_author_slug ] = $current_user->user_login;
        }
    }

    $user_login = (string) sanitize_text_field( $_GET[ $cinnamon_author_slug ] );
    $user_array = $wpdb->get_row( $wpdb->prepare( "SELECT * FROM $wpdb->users WHERE user_nicename = '%s'", $user_login ) );

    if ( empty( $user_array ) ) {
        $user_array = $wpdb->get_row( $wpdb->prepare( "SELECT * FROM $wpdb->users WHERE user_login = '%s'", $user_login ) );
    }

    // If no profile is provided
    if ( empty( $user_login ) || empty( $user_array ) ) {
        return;
    }

    if ( empty( $author) ) {
        $author = $user_array->ID;
    }
    if ( empty( $username ) ) {
        $username = $user_array->user_nicename;
    }

    $imagepress_slug = imagepress_get_option( 'ip_slug' );
    $hub_user_info   = get_userdata( (int) $author );

    wp_enqueue_style( 'icofont' );

    $display = '';

    $hub_twitter   = '';
    $hub_instagram = '';
    $hub_linkedin  = '';
    $hub_facebook  = '';
    $hub_pinterest = '';
    $hub_user_url  = '';

    $hub_twitter   = (string) $hub_user_info->twitter !== '' ? '<a href="https://twitter.com/' . $hub_user_info->twitter . '" target="_blank"><i class="icofont-twitter"></i></a>' : '';
    $hub_linkedin  = (string) $hub_user_info->linkedin !== '' ? '<a href="' . $hub_user_info->linkedin . '" target="_blank"><i class="icofont-linkedin"></i></a>' : '';
    $hub_instagram = (string) $hub_user_info->instagram !== '' ? '<a href="https://instagram.com/' . $hub_user_info->instagram . '" target="_blank"><i class="icofont-instagram"></i></a>' : '';
    $hub_facebook  = (string) $hub_user_info->facebook !== '' ? '<a href="' . $hub_user_info->facebook . '" target="_blank"><i class="icofont-facebook"></i></a>' : '';
    $hub_pinterest = (string) $hub_user_info->pinterest !== '' ? '<a href="https://pinterest.com/' . $hub_user_info->pinterest . '" target="_blank"><i class="icofont-pinterest"></i></a>' : '';
    $hub_user_url  = (string) $hub_user_info->user_url !== '' ? '<a href="' . $hub_user_info->user_url . '" rel="external" target="_blank"><i class="icofont-link"></i></a>' : '';

    $hub_links = $hub_twitter . $hub_instagram . $hub_linkedin . $hub_facebook . $hub_pinterest . $hub_user_url;

    $hca = get_the_author_meta( 'hub_custom_cover', $author );
    $hca = wp_get_attachment_url( $hca );

    if ( (int) get_the_author_meta( 'cover_height', $userid ) > 0 ) {
        $cinnamon_cover_height = (int) get_the_author_meta( 'cover_height', $userid );
    } elseif ( (int) get_option( 'cinnamon_cover_height' ) > 0 ) {
        $cinnamon_cover_height = (int) get_option( 'cinnamon_cover_height' );
    } else {
        $cinnamon_cover_height = 300;
    }

    if ( ! isset( $hca ) || empty( $hca ) ) {
        $hca = IMAGEPRESS_PLUGIN_URL . '/img/coverimage.png';
    }

    $logged_in_user = wp_get_current_user();

    $display .= '<div class="profile-hub-container">';

        if ( imagepress_get_option( 'cinnamon_fancy_header' ) === 'yes' ) {
            $alignfull = ( (int) get_option( 'cinnamon_fullwidth_header' ) === 1 ) ? 'alignfull' : '';

            $display .= '<div class="cinnamon-cover ' . $alignfull . '" data-background-image="' . $hca . '" style="--cinnamon-cover-height:' . $cinnamon_cover_height . 'px;background:url(' . $hca . ') no-repeat center center / cover;">
                <div class="cinnamon-cover--inner wrap-inner">';

                    $display .= '<div class="cinnamon-avatar">
                        <div class="cinnamon-user">' . get_avatar( $author, 120 );

                            if ( is_user_logged_in() && $username != $logged_in_user->user_login ) {
                                $display .= '<div class="imagepress-follow">' . do_shortcode( '[follow_links follow_id="' . $author . '"]' ) . '</div>';
                            }

                        $display .= '</div>';

                        // Get custom URL
                        //$hubdomain = preg_replace('/^www\./', '', $_SERVER['HTTP_HOST']);
                        $hubuser  = get_user_by( 'id', $author );
                        $hubuser  = sanitize_title( $hubuser->user_login );
                        $hub_name = $hub_user_info->first_name . ' ' . $hub_user_info->last_name;
                        if ( empty( $hub_user_info->first_name ) && empty( $hub_user_info->last_name ) ) {
                            $hub_name = $hubuser;
                        }

                        $hub_anchor = imagepress_get_profile_uri( $author, false );
                        $cover_text_color = get_user_meta( $author, 'cover_text_color', true ) !== '' ? get_user_meta( $author, 'cover_text_color', true ) : '#ffffff';

                        $display .= '<div>
                            <div class="ph-nametag" style="color:' . $cover_text_color . '">
                                ' . $hub_name;

                                if ( is_user_logged_in() ) {
                                    if ( (int) $author === (int) $logged_in_user->ID ) {
                                        $display .= ' <a href="' . $hub_anchor . '">#</a>';
                                        $display .= ' <small><a href="' . get_permalink(imagepress_get_option('cinnamon_edit_page')) . '">' . __('Edit Profile', 'imagepress') . '</a></small>';
                                    }
                                }

                                if ( (string) get_user_meta( $author, 'imagepress_role', true ) !== '' ) {
                                    $display .= '<br><small style="color:' . $cover_text_color . '">' . get_user_meta( $author, 'imagepress_role', true ) . '</small>';
                                }

                                /*
                                if ( is_user_logged_in() && (string) strtolower( $username ) === (string) strtolower( $logged_in_user->user_login ) ) {
                                    $display .= ' <a href="' . $hub_anchor . '">#</a>';
                                    $display .= ' <small><a href="' . get_permalink(imagepress_get_option('cinnamon_edit_page')) . '">' . __('Edit Profile', 'imagepress') . '</a></small>';
                                }
                                /**/
                            $display .= '</div>
                            <div class="ph-locationtag">' . $hub_links . '</div>
                        </div>';
                    $display .= '</div>
                </div>';
            $display .= '</div>';
        } else {
            $display .= '<div class="cinnamon-cover-blank">
                <p>' . get_avatar($author, 60) . '</p>';
                if (is_user_logged_in() && (string) $username !== (string) $logged_in_user->user_login) {
                    $display .= '<div class="imagepress-follow">' . do_shortcode('[follow_links follow_id="' . $author . '"]') . '</div>';
                }

                // get custom URL
                //$hubdomain = preg_replace('/^www\./', '', $_SERVER['HTTP_HOST']);
                $hubuser = get_user_by('id', $author);
                $hubuser = sanitize_title($hubuser->user_login);
                $hub_name = $hub_user_info->first_name . ' ' . $hub_user_info->last_name;
                if (empty($hub_user_info->first_name) && empty($hub_user_info->last_name)) {
                    $hub_name = $hubuser;
                }

                $display .= '<h2>' . $hub_name . '</h2>
                <p class="ph-locationtag">';
                    $display .= $hub_links;

                    if (is_user_logged_in() && $username == $logged_in_user->user_login) {
                        $display .= ' | <a href="' . get_permalink(imagepress_get_option('cinnamon_edit_page')) . '">' . __('Edit Profile', 'imagepress') . '</a>';
                    }
                $display .= '</p>
            </div>';
        }

        // Cinnamon Stats
        $last_login                  = get_user_meta( $author, 'imagepress_last_login', true );
        $last_login_formatted        = date( get_option( 'date_format' ), strtotime( $last_login ) );
        $registration_date           = $hub_user_info->user_registered;
        $registration_date_formatted = date( get_option( 'date_format' ), strtotime( $registration_date ) );

        $display .= '<div class="cinnamon-stats">
            <div class="cinnamon-stats--time">
                <i class="icofont-clock-time ui--d"></i> ' . __( 'Last seen online on', 'imagepress' ) . ' ' . $last_login_formatted . ' (' . abs( round( ( ( ( time() - strtotime( $last_login ) ) / 60 ) / 60 ) / 24 ) ) . ' ' . __( 'day(s) ago', 'imagepress' ) . ')
                <br><small class="ui--d">' . __( 'Joined on', 'imagepress' ) . ' ' . $registration_date_formatted . '</small>';

                $award_terms = wp_get_object_terms( $author, 'award' );

                if ( ! empty( $award_terms ) ) {
                    if ( ! is_wp_error( $award_terms ) ) {
                        $display .= '<br>';

                        foreach ( $award_terms as $term ) {
                            $t_ID      = $term->term_id;
                            $term_data = get_option( "taxonomy_$t_ID" );

                            $display .= '<small title="' . $term->description . '"><i class="icofont-badge"></i> <span class="ui--d">' . $term->name . '</span> </small> ';
                        }
                    }
                }

            $display .= '</div>
            <div class="ui-flex--row">
                <div class="cinnamon-meta"><b>' . imagepress_kformat( imagepress_post_views( $author ) ) . '</b> <span class="ui--d">' . __( 'profile views', 'imagepress' ) . '</span></div>
                <div class="cinnamon-meta"><b>' . imagepress_kformat( imagepress_get_follower_count( $author ) ) . '</b> <span class="ui--d">' . __( 'followers', 'imagepress' ) . '</span></div>
                <div class="cinnamon-meta"><b>' . imagepress_kformat( imagepress_count_user_posts_by_type( $author, $imagepress_slug ) ) . '</b> <span class="ui--d">' . __( 'uploads', 'imagepress' ) . '</span></div>
            </div>
        </div>';

        $display .= '<div class="ip-profile">';

            if ( ! empty( $hub_user_info->description ) ) {
                $display .= '<div class="ip-profile--bio">' . wpautop( $hub_user_info->description ) . '</div>';
            }

            $display .= do_shortcode( '[imagepress-loop user="' . $author . '" count="999999"]' ) .
        '</div>
    </div>';

    return $display;
}



function imagepress_profile_edit( $atts ) {
    $atts = shortcode_atts(
        [
            'author' => 0,
        ],
        $atts
    );

    $author = $atts['author'];

    global $wpdb;

    wp_enqueue_style( 'icofont' );

    $current_user = wp_get_current_user();
    $userid       = $current_user->ID;

    $error = [];
    $out   = '';

    if ( ! empty( $_POST['action'] ) && (string) $_POST['action'] === 'update-user' ) {
        if ( ! empty( $_POST['pass1'] ) && ! empty( $_POST['pass2'] ) ) {
            if ( (string) $_POST['pass1'] === (string) $_POST['pass2'] ) {
                wp_update_user(
                    [
                        'ID'        => $userid,
                        'user_pass' => esc_attr( $_POST['pass1'] ),
                    ]
                );
            } else {
                $error[] = __( 'The passwords you entered do not match. Your password was not updated.', 'imagepress' );
            }
        }

        wp_update_user(
            [
                'ID'       => $userid,
                'user_url' => esc_url( sanitize_url( $_POST['url'] ) ),
            ]
        );

        if ( ! is_email( esc_attr( $_POST['email'] ) ) )  {
            $error[] = __( 'The email you entered is not valid. Please try again.', 'imagepress' );
        } elseif ( email_exists( esc_attr( $_POST['email'] ) ) != $userid ) {
            $error[] = __( 'This email is already used by another user. Try a different one.', 'imagepress' );
        } else {
            wp_update_user(
                [
                    'ID'         => $userid,
                    'user_email' => esc_attr( sanitize_email( $_POST['email'] ) ),
                ]
            );
        }

        update_user_meta( $userid, 'first_name', esc_attr( sanitize_text_field( $_POST['first-name'] ) ) );
        update_user_meta( $userid, 'last_name', esc_attr( sanitize_text_field( $_POST['last-name'] ) ) );

        update_user_meta( $userid, 'nickname', esc_attr( sanitize_text_field( $_POST['nickname'] ) ) );
        $wpdb->update(
            $wpdb->users, [
                'display_name' => $_POST['nickname'],
            ],
            [
                'ID' => $userid,
            ],
            null,
            null
        );

        update_user_meta($userid, 'description', esc_attr( sanitize_text_field( $_POST['description'] ) ) );

        update_user_meta( $userid, 'twitter', esc_attr( $_POST['twitter'] ) );
        update_user_meta( $userid, 'instagram', esc_attr( $_POST['instagram'] ) );
        update_user_meta( $userid, 'linkedin', esc_attr( $_POST['linkedin'] ) );
        update_user_meta( $userid, 'facebook', esc_attr( $_POST['facebook'] ) );
        update_user_meta( $userid, 'pinterest', esc_attr( $_POST['pinterest'] ) );

        update_user_meta( $userid, 'imagepress_role', sanitize_text_field( $_POST['role'] ) );

        // Profile settings
        update_user_meta( $userid, 'cover_text_color', sanitize_text_field( $_POST['cover_text_color'] ) );
        update_user_meta( $userid, 'cover_height', (int) sanitize_text_field( $_POST['cover_height'] ) );
        //

        // Avatar and cover upload
        if ( $_FILES ) {
            require_once ABSPATH . '/wp-admin/includes/image.php';
            require_once ABSPATH . '/wp-admin/includes/file.php';
            require_once ABSPATH . '/wp-admin/includes/media.php';

            foreach ( $_FILES as $file => $array ) {
                if ( ! empty( $_FILES[ $file ]['name'] ) ) {
                    $file_id = media_handle_upload( $file, 0 );

                    if ( $file_id > 0 ) {
                        update_user_meta( $userid, $file, $file_id );
                    }
                }
            }
        }
        //

        if ( count( $error ) === 0 ) {
            //do_action('edit_user_profile_update', $userid);
            $out .= '<p class="message noir-success">' . __( 'Profile updated successfully!', 'imagepress' ) . '</p>';
        }
    }

    $out .= '<div id="post-' . get_the_ID() . '">
        <div class="entry-content entry cinnamon cinnamon-form">';

            if ( count( $error ) > 0 ) {
                $out .= '<p class="error">' . implode( '<br>', $error ) . '</p>';
            }

            $out .= '<form method="post" id="adduser" action="" enctype="multipart/form-data">
                <ul class="imagepress--tabs">
                    <li><a href="#summary" class="is-active">' . __( 'Summary', 'imagepress' ) . '</a></li>
                    <li><a href="#account">' . __( 'Account Details', 'imagepress' ) . '</a></li>
                    <li><a href="#profile">' . __( 'Profile Settings', 'imagepress' ) . '</a></li>
                    <li><a href="#collections" class="imagepress-collections">' . __( 'Collections', 'imagepress' ) . '</a></li>
                    <li><a href="#editor">' . __( 'Image Editor', 'imagepress' ) . '</a></li>
                </ul>
                <div class="imagepress--tab" id="summary">
                    <h3>' . __( 'Statistics', 'imagepress' ) . '</h3>';
                    $imagepress_slug = imagepress_get_option( 'ip_slug' );

                    // get current user uploads
                    $user_uploads = imagepress_count_user_posts_by_type($userid, $imagepress_slug);

                    $out .= '<div class="ip-user-dashboard">
                        <div class="ip-user-dashboard-stat">
                            <span>' . imagepress_kformat(imagepress_post_views($userid, false)) . '</span>' .
                            __('total profile views', 'imagepress') . '
                        </div>
                        <div class="ip-user-dashboard-stat">
                            <span>' . imagepress_kformat(imagepress_get_follower_count($userid)) . '</span>' .
                            __('followers', 'imagepress') . '
                        </div>
                        <div class="ip-user-dashboard-stat">
                            <span>' . imagepress_kformat(imagepress_get_following_count($userid)) . '</span>' .
                            __('following', 'imagepress') . '
                        </div>
                        <div class="ip-user-dashboard-stat">
                            <span>' . imagepress_kformat(imagepress_count_user_posts_by_type($userid, $imagepress_slug)) . '</span>' .
                            __('uploads', 'imagepress') . '
                        </div>';

                        $out .= '<div class="ip-user-dashboard-stat">
                            <span>' . imagepress_kformat(imagepress_collection_count($userid)) . '</span>' .
                            __('collections', 'imagepress') . '
                        </div>';

                    $out .= '</div>';
                    $out .= '<div class="ip_clear"></div>';

                    // Followers
                    $arr = imagepress_get_followers( $userid );

                    if ( $arr ) {
                        $out .= '<h3>' . __( 'Followers', 'imagepress' ) . '</h3>';

                        $out .= '<div class="cinnamon-followers">';

                        foreach ( $arr as $value ) {
                                $user = get_user_by( 'id', $value );
                                $out .= '<a href="' . imagepress_get_profile_uri( $value, false ) . '">' . get_avatar( $value, 40 ) . '</a> ';
                        }
                        unset( $value );

                        $out .= '</div>';
                    }

                    // Following
                    $arr = imagepress_get_following( $userid );

                    if ($arr) {
                        $out .= '<h3>' . __( 'Following', 'imagepress' ) . '</h3>';

                        $out .= '<div class="cinnamon-followers">';

                        foreach ( $arr as $value ) {
                            $user = get_user_by( 'id', $value );
                            $out .= '<a href="' . imagepress_get_profile_uri( $value, false ) . '">' . get_avatar( $value, 40 ) . '</a> ';
                        }
                        unset( $value );

                        $out .= '</div>';
                    }

                    $out .= imagepress_frontend_user_likes( $userid );

                $out .= '</div>

                <div class="imagepress--tab" id="account">
                    <h3>' . __( 'Personal Details', 'imagepress' ) . '</h3>
 
                    <div class="imagepress--tab-element">
                        <label for="first-name">' . __( 'First name', 'imagepress' ) . '</label>
                        <input name="first-name" type="text" id="first-name" value="' . get_the_author_meta( 'first_name', $userid ) . '">
                    </div>
                    <div class="imagepress--tab-element">
                        <label for="last-name">' . __(' Last name', 'imagepress' ) . '</label>
                        <input name="last-name" type="text" id="last-name" value="' . get_the_author_meta( 'last_name', $userid ) . '">
                    </div>
                    <div class="imagepress--tab-element">
                        <label for="nickname">' . __( 'Nickname', 'imagepress' ) . '</label>
                        <input name="nickname" type="text" id="nickname" value="' . get_the_author_meta( 'nickname', $userid ) . '">
                    </div>
                    <div class="imagepress--tab-element">
                        <label for="email">' . __( 'Email *', 'imagepress' ) . '</label>
                        <input name="email" type="text" id="email" value="' . get_the_author_meta( 'user_email', $userid ) . '">
                    </div>
                    <div class="imagepress--tab-element">
                        <label for="url">' . __( 'Website', 'imagepress' ) . '</label>
                        <input name="url" type="text" id="url" value="' . get_the_author_meta( 'user_url', $userid ) . '">
                    </div>
                    <div class="imagepress--tab-element">
                        <label for="pass1">' . __('Password *', 'imagepress') . '</label>
                        <input name="pass1" type="password" id="pass1">
                    </div>
                    <div class="imagepress--tab-element">
                        <label for="pass2">' . __('Repeat password *', 'imagepress') . '</label>
                        <input name="pass2" type="password" id="pass2">
                    </div>

                    <h3>' . __( 'Professional Details', 'imagepress' ) . '</h3>

                    <div class="imagepress--tab-element">
                        <label for="role">' . __( 'Role', 'imagepress' ) . '</label>
                        <input name="role" type="text" id="role" value="' . get_user_meta( $userid, 'imagepress_role', true ) . '">
                    </div>
                    <div class="imagepress--tab-element">
                        <label for="description">' . __('About', 'imagepress') . '</label>
                        <textarea name="description" id="description" rows="4">' . get_the_author_meta('description', $userid) . '</textarea>
                    </div>

                    <h3>' . __( 'Social Details', 'imagepress' ) . '</h3>

                    <div class="imagepress--tab-element">
                        <label for="facebook">' . __('Facebook profile URL', 'imagepress') . '</label>
                        <input name="facebook" type="url" id="facebook" value="' . get_the_author_meta('facebook', $userid) . '">
                    </div>
                    <div class="imagepress--tab-element">
                        <label for="twitter">' . __('X username', 'imagepress') . '</label>
                        <input name="twitter" type="text" id="twitter" value="' . get_the_author_meta('twitter', $userid) . '">
                    </div>
                    <div class="imagepress--tab-element">
                        <label for="instagram">' . __('Instagram username', 'imagepress') . '</label>
                        <input name="instagram" type="text" id="instagram" value="' . get_the_author_meta('instagram', $userid) . '">
                    </div>
                    <div class="imagepress--tab-element">
                        <label for="linkedin">' . __('LinkedIn profile URL', 'imagepress') . '</label>
                        <input name="linkedin" type="url" id="linkedin" value="' . get_the_author_meta('linkedin', $userid) . '">
                    </div>
                    <div class="imagepress--tab-element">
                        <label for="pinterest">' . __('Pinterest username', 'imagepress') . '</label>
                        <input name="pinterest" type="text" id="pinterest" value="' . get_the_author_meta('pinterest', $userid) . '">
                    </div>
                </div>';

                $cinnamon_cover_height = (int) get_the_author_meta( 'cover_height', $userid ) > 0 ? (int) get_the_author_meta( 'cover_height', $userid ) : 300;

                $out .= '<div class="imagepress--tab" id="profile">
                    <div class="imagepress--tab-element">';

                        if ( ! is_admin() ) {
                            $out .= '<label>' . __('Cover/avatar preview', 'imagepress') . '</label>';

                            $hcc = get_the_author_meta( 'hub_custom_cover', $userid );
                            $hca = get_the_author_meta( 'hub_custom_avatar', $userid );
                            $hcc = wp_get_attachment_url( $hcc );
                            $hca = wp_get_attachment_url( $hca );

                            $out .= '<div class="cinnamon-cover-preview" data-background="' . $hcc . '"><img src="' . $hca . '" alt=""></div>';
                        }

                    $out .= '</div>';

                    $out .= '<div class="imagepress--tab-element">
                        <label for="hub_custom_cover">' . __('Profile cover image', 'imagepress') . '</label>
                        <div>
                            <input type="file" name="hub_custom_cover" id="hub_custom_cover" value="' . get_the_author_meta('hub_custom_cover', $userid) . '" class="regular-text">
                            <br><small>' . __('Minimum recommended cover size is 1080x600. ', 'imagepress') . '</small>
                        </div>
                    </div>
                    <div class="imagepress--tab-element">
                        <label for="hub_custom_avatar">' . __('Profile avatar image', 'imagepress') . '</label>
                        <div>
                            <input type="file" name="hub_custom_avatar" id="hub_custom_avatar" value="' .  get_the_author_meta('hub_custom_avatar', $userid) . '" class="regular-text">
                            <br><small>' . __('Minimum recommended avatar size is 512x512. If there is no custom avatar, your Gravatar will be used.', 'imagepress') . '</small>
                        </div>
                    </div>

                    <div class="imagepress--tab-element">
                        <label for="cover-text-color">' . __( 'Cover text colour', 'imagepress' ) . '</label>
                        <input name="cover_text_color" type="color" list="colour-tokens-cover-text" id="cover-text-color" value="' . get_the_author_meta( 'cover_text_color', $userid ) . '">
                        <datalist id="colour-tokens-cover-text">
                            <option>#ffffff</option>
                            <option>#f9f9f9</option>
                            <option>#ececec</option>
                            <option>#dedede</option>
                            <option>#cecece</option>
                            <option>#bababa</option>
                            <option>#a0a0a0</option>
                            <option>#848484</option>
                            <option>#6c6c6c</option>
                            <option>#595959</option>
                            <option>#3f3f3f</option>
                            <option>#272727</option>
                            <option>#000000</option>
                        </datalist>
                    </div>
                    <div class="imagepress--tab-element">
                        <label for="cover-height">' . __( 'Cover height', 'imagepress' ) . '</label>
                        <div style="display:flex;gap:16px;align-items:center">
                            <input type="range" list="cover-height-tokens" min="100" max="600" step="2" value="' . $cinnamon_cover_height . '" step="10" name="cover_height" id="cover-height" style="width:240px">
                            <datalist id="cover-height-tokens">
                                <option>100</option>
                                <option>200</option>
                                <option>300</option>
                                <option>400</option>
                                <option>500</option>
                                <option>600</option>
                            </datalist>

                            <small class="imagepress--tab-value"><code id="cover-height-hint"></code>px</small>
                            <script>
                            let slider = document.getElementById("cover-height");
                            let output = document.getElementById("cover-height-hint");
                            output.innerHTML = slider.value;

                            slider.oninput = () => {
                                output.innerHTML = slider.value;
                            }
                            </script>
                        </div>
                    </div>
                </div>';


                $out .= '<div class="imagepress--tab" id="collections">
                    <p>
                        <a href="#" class="toggleModal button">' . __('Create new collection', 'imagepress') . '</a>
                        <span class="ip-loadingCollections">' . __('Loading collections...', 'imagepress') . '</span>
                        <span class="ip-loadingCollectionImages">' . __('Loading collection images...', 'imagepress') . '</span>
                    </p>
                    <div class="ip-modal">
                        <h2>' . __('Create new collection', 'imagepress') . '</h2>
                        <a href="#" class="close toggleModal">' . __('Close', 'imagepress') . '</a>

                        <input type="hidden" id="collection_author_id" name="collection_author_id" value="' . $userid . '">
                        <p><input type="text" id="collection_title" name="collection_title" placeholder="' . __('Collection title', 'imagepress') . '"></p>
                        <p><label>Make this collection</label> <select id="collection_status"><option value="1">' . __('Public', 'imagepress') . '</option><option value="0">' . __('Private', 'imagepress') . '</option></select></p>
                        <p>
                            <input type="submit" value="' . __('Create', 'imagepress') . '" class="addCollection">
                            <label class="collection-progress">' . __('Creating collection...', 'imagepress') . '</label>
                            <label class="showme">' . __('Collection created!', 'imagepress') . '</label>
                        </p>
                    </div>

                    <div class="collections-display"></div>
                </div>';

                // Image Editor
                // View, delete, reorder
                $out .= '<div class="imagepress--tab" id="editor">
                    <div id="ip-info">' . __('Drag images to reorder them', 'imagepress') . '<br><small>' . __('Click titles to rename images', 'imagepress') . '</small></div>';

                    $args = [
                        'post_type' => imagepress_get_option('ip_slug'),
                        'post_status' => ['publish', 'pending'],
                        'posts_per_page' => '-1',
                        'orderby' => 'menu_order',
                        'order' => 'ASC',
                        'author' => $userid,
                        'cache_results' => false,
                        'no_found_rows' => true
                    ];
                    $posts = get_posts($args);

                    $ip_click_behaviour = imagepress_get_option('ip_click_behaviour');

                    $out .= '<div class="editor-image-manager">';
                        if ($posts) {
                            foreach($posts as $user_image) {
                                $i = $user_image->ID;

                                $post_thumbnail_id = get_post_thumbnail_id($i);
                                $image_attributes = wp_get_attachment_image_src($post_thumbnail_id, 'thumbnail');

                                if ($ip_click_behaviour == 'media')
                                    $ip_image_link = $image_attributes[0];
                                if ($ip_click_behaviour == 'custom')
                                    $ip_image_link = get_permalink($i);

                                $out .= '<div class="editor-image ip_box_' . $i . '" id="listItem_' . $i . '" data-id="' . $i . '">
                                    <div class="editor-image-handle"><i class="icofont-drag"></i></div>
                                    <div class="editor-image-thumbnail">
                                        <a href="' . $ip_image_link . '"><img src="' . $image_attributes[0] . '" alt="' . get_the_title($i) . '"></a>
                                    </div>
                                    <div>
                                        <input type="text" class="editableImage" id="listImage_' . $i . '" data-image-id="' . $i . '" value="' . get_the_title($i) . '">
                                        <span class="editableImageStatus editableImageStatus_' . $i . '"></span>
                                        <br><small>' . __('in', 'imagepress') . ' ' .  strip_tags(get_the_term_list($i, 'image-category', '', ', ', '')) . ' ' . __('on', 'imagepress') . ' ' . get_the_date('Y-m-d H:i', $i) . '</small>
                                        <br><small><a href="' . $ip_image_link . '">' . __('View/Edit', 'imagepress') . '</a> | <span class="clickable-span" data-image-id="' . $i . '">' . __('Delete', 'imagepress') . '</span></small>
                                    </div>
                                </div>';
                            }
                        }
                    $out .= '</div>';
                $out .= '</div>

                <!-- Begin do_confirm -->
                <div id="aep_ww"><div id="aep_win"><div id="aep_prompt"></div><input type="button" class="aep_ok" value="OK"><input type="button" class="aep_cancel" value="Cancel"></div></div>
                <!-- End do_confirm -->';

                //do_action('edit_user_profile', $current_user);

                wp_nonce_field( 'update-user', '_wpnonce', true, false );

                $out .= '<hr>
                <p>
                    <input name="action" type="hidden" id="action" value="update-user">
                    <input name="updateuser" type="submit" class="button" id="updateuser" value="' . __( 'Update', 'imagepress' ) . '">
                    <a href="' . imagepress_get_profile_uri( $userid, false ) . '">' . __( 'View profile', 'imagepress' ) . '</a>
                </p>
            </form>
        </div>
    </div>';

    if ( ! is_user_logged_in() ) {
        $out = '<p class="warning">' . __( 'You must be logged in to edit your profile.', 'imagepress' ) . '</p>';
    }

    return $out;
}







/* CINNAMON CUSTOM PROFILE FIELDS */
function imagepress_save_award_fields( $user_id ) {
    if ( ! current_user_can( 'edit_user', $user_id ) ) {
        return false;
    }

    // Awards
    if ( current_user_can( 'manage_options', $user_id ) ) {
        if ( ! empty( $_POST['award'] ) ) {
            $term = $_POST['award'];

            wp_set_object_terms( $user_id, $term, 'award', false );
            clean_object_term_cache( $user_id, 'award' );
        }
    }
}

function imagepress_hub_gravatar_filter($avatar, $id_or_email, $size) {
    // do not use email for get_avatar(), use ID
    $current_user = wp_get_current_user();

    $image_url = get_user_meta($id_or_email, 'hub_custom_avatar', true);
    $custom_avatar = wp_get_attachment_url($image_url);

    if (!empty($image_url)) {
        return '<img src="' . $custom_avatar . '" class="avatar" width="' . $size . '" height="' . $size . '" alt="' . $current_user->display_name . '">';
    }

    return $avatar;
}

function imagepress_awards() {
    $args = [
        'hide_empty' => false,
        'pad_counts' => true
    ];
    $terms = get_terms('award', $args);

    if (!empty($terms) && !is_wp_error($terms)) {
        foreach ($terms as $term) {
            $t_ID = $term->term_id;
            $term_data = get_option("taxonomy_$t_ID");

            echo '<p>
                <span title="' . $term->description . '"> ' .
                $term->name . '</span> <span>' . $term->description . '<br><small>(' . $term->count . ' author(s) received this award)</small></span>
            </p>';
        }
    }
}
