<?php
function imagepress_registration() {
    $imagepress_slug = sanitize_text_field( imagepress_get_option( 'ip_slug' ) );

    if ( empty( $imagepress_slug ) ) {
        $imagepress_slug = 'image';
    }

    $image_type_labels = [
        'name'                  => _x( 'Images', 'Post type general name', 'imagepress' ),
        'singular_name'         => _x( 'Image', 'Post type singular name', 'imagepress' ),
        'menu_name'             => __( 'ImagePress', 'imagepress' ),
        'name_admin_bar'        => __( 'Image', 'imagepress' ),
        'archives'              => __( 'Image archives', 'imagepress' ),
        'parent_item_colon'     => __( 'Parent image:', 'imagepress' ),
        'all_items'             => __( 'All images', 'imagepress' ),
        'add_new_item'          => __( 'Add new image', 'imagepress' ),
        'add_new'               => __( 'Add new', 'imagepress' ),
        'new_item'              => __( 'New image', 'imagepress' ),
        'edit_item'             => __( 'Edit image', 'imagepress' ),
        'update_item'           => __( 'Update image', 'imagepress' ),
        'view_item'             => __( 'View image', 'imagepress' ),
        'search_items'          => __( 'Search image', 'imagepress' ),
        'not_found'             => __( 'Not found', 'imagepress' ),
        'not_found_in_trash'    => __( 'Not found in trash', 'imagepress' ),
        'featured_image'        => __( 'Featured image', 'imagepress' ),
        'set_featured_image'    => __( 'Set featured image', 'imagepress' ),
        'remove_featured_image' => __( 'Remove featured image', 'imagepress' ),
        'use_featured_image'    => __( 'Use as featured image', 'imagepress' ),
        'insert_into_item'      => __( 'Insert into image', 'imagepress' ),
        'uploaded_to_this_item' => __( 'Uploaded to this image', 'imagepress' ),
        'items_list'            => __( 'Images list', 'imagepress' ),
        'items_list_navigation' => __( 'Images list navigation', 'imagepress' ),
        'filter_items_list'     => __( 'Filter images list', 'imagepress' ),
    ];

    $image_type_args = [
        'label'                 => __( 'Image', 'imagepress' ),
        'description'           => __( 'Image post type', 'imagepress' ),
        'labels'                => $image_type_labels,
        'supports'              => [ 'title', 'editor', 'author', 'thumbnail', 'comments', 'custom-fields', 'wpcom-markdown' ],
        'hierarchical'          => false,
        'public'                => true,
        'show_ui'               => true,
        'show_in_menu'          => true,
        'menu_position'         => 5,
        'menu_icon'             => 'dashicons-format-gallery',
        'show_in_admin_bar'     => true,
        'show_in_nav_menus'     => true,
        'can_export'            => true,
        'has_archive'           => true,
        'exclude_from_search'   => false,
        'publicly_queryable'    => true,
        'capability_type'       => 'post',
        'show_in_rest'          => true,
        'rest_base'             => $imagepress_slug,
        'rest_controller_class' => 'WP_REST_Posts_Controller',
    ];

    register_post_type( $imagepress_slug, $image_type_args );

    $image_taxonomy = [
        'name'                       => _x( 'Image categories', 'Taxonomy general name', 'imagepress' ),
        'singular_name'              => _x( 'Image category', 'Taxonomy singular name', 'imagepress' ),
        'menu_name'                  => __( 'Image Categories', 'imagepress' ),
        'all_items'                  => __( 'All image categories', 'imagepress' ),
        'parent_item'                => __( 'Parent image category', 'imagepress' ),
        'parent_item_colon'          => __( 'Parent image category:', 'imagepress' ),
        'new_item_name'              => __( 'New image category', 'imagepress' ),
        'add_new_item'               => __( 'Add new image category', 'imagepress' ),
        'edit_item'                  => __( 'Edit image category', 'imagepress' ),
        'update_item'                => __( 'Update image category', 'imagepress' ),
        'view_item'                  => __( 'View image category', 'imagepress' ),
        'separate_items_with_commas' => __( 'Separate image categories with commas', 'imagepress' ),
        'add_or_remove_items'        => __( 'Add or remove image categories', 'imagepress' ),
        'choose_from_most_used'      => __( 'Choose from the most used', 'imagepress' ),
        'popular_items'              => __( 'Popular image categories', 'imagepress' ),
        'search_items'               => __( 'Search image categories', 'imagepress' ),
        'not_found'                  => __( 'Not found', 'imagepress' ),
        'no_terms'                   => __( 'No image categories', 'imagepress' ),
        'items_list'                 => __( 'Image categories list', 'imagepress' ),
        'items_list_navigation'      => __( 'Image categories list navigation', 'imagepress' ),
    ];

    $image_category_args = [
        'labels'                => $image_taxonomy,
        'hierarchical'          => true,
        'public'                => true,
        'show_ui'               => true,
        'show_admin_column'     => true,
        'show_in_nav_menus'     => true,
        'show_tagcloud'         => false,
        'show_in_rest'          => true,
        'rest_base'             => 'image-category',
        'rest_controller_class' => 'WP_REST_Terms_Controller',
    ];

    register_taxonomy( 'image-category', [ $imagepress_slug ], $image_category_args );
}

function imagepress_get_post_views( $post_id ) {
    $count = get_post_meta( $post_id, 'post_views_count', true );
    $count = empty( $count ) ? 0 : $count;

    return number_format( $count );
}
function imagepress_set_post_views( $post_id ) {
    if ( is_singular( imagepress_get_option( 'ip_slug' ) ) && ( ! defined( 'DOING_AJAX' ) || ! DOING_AJAX ) ) {
        error_log( 'Function is running! ' . $post_id );

        $count = get_post_meta( $post_id, 'post_views_count', true );
        $count = empty( $count ) ? 1 : $count + 1;

        update_post_meta( $post_id, 'post_views_count', $count );
    }
}



// frontend image editor
function imagepress_editor() {
    global $wpdb, $post;

    $out = '';

    $current_user = wp_get_current_user();

    // Check if user is author // show author tools
    if ( is_user_logged_in() && (int) $post->post_author > 0 && (int) $post->post_author === (int) $current_user->ID ) {
        $out .= '<a href="#" class="ip-editor-display" id="ip-editor-open">' . __( 'Author tools', 'imagepress' ) . '</a>';

        $edit_id = get_the_ID();

        if ( ! empty( $_POST['post_id'] ) && ! empty( $_POST['post_title'] ) && isset( $_POST['update_post_nonce'] ) && isset( $_POST['postcontent'] ) ) {
            $post_id    = $_POST['post_id'];
            $post_type  = get_post_type( $post_id );
            $capability = ( 'page' === $post_type ) ? 'edit_page' : 'edit_post';

            if ( current_user_can( $capability, $post_id ) && wp_verify_nonce( $_POST['update_post_nonce'], 'update_post_' . $post_id ) ) {
                $post = [
                    'ID'           => esc_sql( $post_id ),
                    'post_content' => ( stripslashes( $_POST['postcontent'] ) ),
                    'post_title'   => esc_sql( $_POST['post_title'] ),
                    'post_name'    => sanitize_text_field( $_POST['post_title'] ),
                ];

                wp_update_post( $post );

                // If an image has been provided, delete the current featured image
                if ( ! empty( $_FILES['imagepress_image_file'] ) ) {
                    $thumbnail_id = get_post_thumbnail_id( $post_id );

                    if ( $thumbnail_id ) {
                        // Delete the current featured image
                        delete_post_thumbnail( $post_id );

                        // Delete the old image completely
                        wp_delete_attachment( $thumbnail_id, true );
                    }

                    imagepress_process_image( 'imagepress_image_file', $post_id );
                }

                // Multiple images
                imagepress_upload_secondary( $_FILES['imagepress_image_additional'], $post_id );

                wp_set_object_terms( $post_id, (int) $_POST['imagepress_image_category'], 'image-category' );
            }
        }

        $out .= '<div id="info" class="ip-editor">
            <form id="post" class="post-edit front-end-form imagepress-form" method="post" enctype="multipart/form-data">
                <input type="hidden" name="post_id" value="' . $edit_id . '">';
                $out .= wp_nonce_field( 'update_post_' . $edit_id, 'update_post_nonce', true, false );

                $out .= '<p>
                    <label for="post_title">' . __( 'Title', 'imagepress' ) . '</label><br>
                    <input type="text" id="post_title" name="post_title" value="' . get_the_title( $edit_id ) . '">
                </p>
                <p>
                    <label for="postcontent">' . __( 'Description', 'imagepress' ) . '</label><br>
                    <textarea id="postcontent" name="postcontent" rows="3">' . strip_tags( get_post_field( 'post_content', $edit_id ) ) . '</textarea></p>
                <hr>';

                $out .= '<hr>';

                $ip_category = wp_get_object_terms( $edit_id, 'image-category' );

                $out .= imagepress_get_image_categories_dropdown( 'image-category', $ip_category[0]->term_id );

                $ip_upload_size = imagepress_get_option( 'ip_upload_size' );
                $uploadsize     = number_format( ( ( $ip_upload_size * 1024 ) / 1024000 ), 0, '.', '' );
                $datauploadsize = $uploadsize * 1024000;

                //
                $out .= '<hr>

                <label for="imagepress_image_file" id="dropContainer" class="dropSelector">
                    <b>' . __( 'Drop files here<br><small>or</small>', 'imagepress' ) . '</b><br>
                    <input type="file" accept="image/*" data-max-size="' . $datauploadsize . '" name="imagepress_image_file" id="imagepress_image_file" required>
                    <br><small>' . $uploadsize . 'MB ' . __( 'maximum', 'imagepress' ) . '</small>
                </label>

                <hr>';
                //

                if ( (int) imagepress_get_option( 'ip_upload_secondary' ) === 1 ) {
                    $out .= '<hr>
                    <p>' .
                        __( 'Delete additional images', 'imagepress' ) .
                    '</p>';

                    $thumbnail_id = get_post_thumbnail_id();
                    $images       = get_children( ['post_parent' => $edit_id, 'post_status' => 'inherit', 'post_type' => 'attachment', 'post_mime_type' => 'image', 'order' => 'ASC', 'orderby' => 'menu_order ID' ] );
                    $count        = $images ? count( $images ) : 0;

                    if ( $count >= 1 ) {
                        $out .= '<div>';
                            foreach ( $images as $attachment_id => $image ) {
                                $small_array = image_downsize( $image->ID, 'thumbnail' );

                                if ( (int) $image->ID !== (int) $thumbnail_id ) {
                                    $out .= '<div class="ip-additional ip-additional-' . $image->ID . '">
                                        <div class="ip-toolbar">
                                            <a href="#" data-image-id="' . $image->ID . '" data-redirect="' . get_permalink() . '" class="ip-delete-post ip-action-icon ip-floatright"><i class="icofont-ui-delete"></i></a>
                                        </div>
                                        <img src="' . $small_array[0] . '" alt="">
                                    </div>';
                                }
                            }
                        $out .= '</div>';
                    }

                    $out .= '<p><label for="imagepress_image_additional">' . __( 'Add more images', 'imagepress' ) . ' (' . $uploadsize . 'MB ' . __( 'maximum', 'imagepress' ) . ')...</label><br><input type="file" accept="image/*" data-max-size="' . $datauploadsize . '" name="imagepress_image_additional[]" id="imagepress_image_additional" multiple></p>';
                }

                $out .= '<hr>';

                $out .= '<p>
                    <input type="submit" id="submit" value="' . __( 'Update', 'imagepress' ) . '">
                </p>
            </form>
        </div>';

        wp_reset_query();

        return $out;
    }
}

// imagepress_editor() related actions
add_action( 'wp_ajax_imagepress_delete_post', 'imagepress_delete_post' );

function imagepress_delete_post() {
    $image_id = (int) $_POST['id'];

    if ( wp_delete_post( $image_id ) ) {
        echo 'success';
    }

    die();
}

add_action( 'wp_ajax_imagepress_update_post_title', 'imagepress_update_post_title' );

function imagepress_update_post_title() {
    $updated_post = [
        'ID'         => (int) $_REQUEST['id'],
        'post_title' => (string) $_REQUEST['title'],
    ];

    wp_update_post( $updated_post );

    echo 'success';

    die();
}



// Main ImagePress image function
function imagepress_main( $image_id ) {
    global $wpdb, $post;

    wp_enqueue_style( 'icofont' );

    $image_size_single  = get_option( 'image_size_single', 'full' );
    $post_thumbnail_id  = get_post_thumbnail_id( $image_id );
    $image_attributes   = wp_get_attachment_image_src( $post_thumbnail_id, $image_size_single );
    $post_thumbnail_url = $image_attributes[0];

    $ip_comments = '';

    if ( (int) imagepress_get_option( 'ip_comments' ) === 1 ) {
        $ip_comments = '<em> | </em><a href="' . get_permalink( $image_id ) . '"><i class="icofont-speech-comments"></i> ' . get_comments_number( $image_id ) . '</a> ';
    }

    // Guest uploads require a name and an email
    if ( (int) $post->post_author > 0 ) {
        $imagepress_image_author_name = imagepress_get_profile_uri( $post->post_author );
    } elseif ( (int) imagepress_get_option( 'ip_registration' ) === 1 && get_post_meta( $image_id, 'imagepress_image_author_name', true ) !== '' ) {
        $imagepress_image_author_name = get_post_meta( $image_id, 'imagepress_image_author_name', true );
    } else {
        $imagepress_image_author_name = __( 'Anonymous', 'imagepress' );
    }

    imagepress_set_post_views( $image_id );
    ?>

    <div class="imagepress-main--single">
        <div>
            <div class="ip-bar">
                <?php echo imagepress_editor(); ?>
            </div>

            <div class="imagepress-container">
                <a href="<?php echo $post_thumbnail_url; ?>">
                    <?php the_post_thumbnail( $image_size_single ); ?>
                </a>
            </div>

            <div class="ip-hooks">
                <?php do_action( 'imagepress_main_content', $image_id ); ?>
            </div>

            <?php imagepress_get_images( $image_id, 1 ); ?>

            <section role="navigation">
                <?php previous_post_link( '%link', __( 'Previous', 'imagepress' ) ); ?>
                &nbsp;|&nbsp;
                <?php next_post_link( '%link', __( 'Next', 'imagepress' ) ); ?>
            </section>
        </div>
        <div class="imagepress--image-sidebar">
            <p>
                <?php echo get_avatar( $post->post_author, 80 ); ?><br>
                <?php echo $imagepress_image_author_name; ?>
                <?php
                if ( (string) get_user_meta( $post->post_author, 'imagepress_role', true ) !== '' ) {
                    echo '<br><small>' . get_user_meta( $post->post_author, 'imagepress_role', true ) . '</small>';
                }
                ?>

                <br><small><?php _e( 'Uploaded', 'imagepress' ); ?> <time title="<?php the_time( get_option( 'date_format' ) ); ?>"><?php echo human_time_diff( get_the_time( 'U' ), current_time( 'timestamp' ) ) . ' ago'; ?></time> <?php _e( 'in', 'imagepress' ); ?> <?php echo get_the_term_list( get_the_ID(), 'image-category', '', ', ', '' ); ?></small>
            </p>

            <div class="ip-bar">
                <?php echo imagepress_get_post_like_link( $image_id ); ?><em> | </em>
                <i class="icofont-eye-alt"></i> <?php echo imagepress_get_post_views( $image_id ); ?>
                <?php echo $ip_comments; ?>
                <em> | </em>
                <?php
                if ( function_exists( 'imagepress_frontend_add_collection' ) ) {
                    echo imagepress_frontend_add_collection( $image_id );
                }
                ?>
            </div>

            <div class="ip-hooks">
                <?php do_action( 'imagepress_sidebar_content', $image_id ); ?>
            </div>

            <?php if ( (int) get_option( 'image_sidebar_pattern_id' ) > 0 ) { ?>
                <div class="imagepress--image-sidebar-patern" data-id="<?php echo get_option( 'image_sidebar_pattern_id' ); ?>">
                    <?php
                    $image_sidebar_pattern_id = get_post( get_option( 'image_sidebar_pattern_id' ) );

                    echo apply_filters( 'the_content', $image_sidebar_pattern_id->post_content );
                    ?>
                </div>
            <?php } ?>

            <h1 class="ip-title">
                <?php echo get_the_title( $image_id ); ?>
            </h1>

            <?php echo wpautop( $post->post_content ); ?>

            <?php
            comments_template( '', true );
            ?>
        </div>
    </div>
    <?php
}



// main ImagePress image function
function imagepress_main_return( $image_id ) {
    global $wpdb, $post;

    wp_enqueue_style( 'icofont' );

    $image_size_single  = get_option( 'image_size_single', 'full' );
    $post_thumbnail_id  = get_post_thumbnail_id( $image_id );
    $image_attributes   = wp_get_attachment_image_src( $post_thumbnail_id, $image_size_single );
    $post_thumbnail_url = '';
    if ( is_array( $image_attributes ) ) {
        $post_thumbnail_url = $image_attributes[0];
    }

    $ip_comments = '';

    if ( (int) imagepress_get_option( 'ip_comments' ) === 1 ) {
        $ip_comments = '<em> | </em><a href="' . get_permalink( $image_id ) . '"><i class="icofont-speech-comments"></i> ' . get_comments_number( $image_id ) . '</a> ';
    }

    // Guest uploads require a name and an email
    if ( (int) $post->post_author > 0 ) {
        $imagepress_image_author_name = imagepress_get_profile_uri( $post->post_author );
    } elseif ( (int) imagepress_get_option( 'ip_registration' ) === 1 && get_post_meta( $image_id, 'imagepress_image_author_name', true ) !== '' ) {
        $imagepress_image_author_name = get_post_meta( $image_id, 'imagepress_image_author_name', true );
    } else {
        $imagepress_image_author_name = __( 'Anonymous', 'imagepress' );
    }

    imagepress_set_post_views( $image_id );

    $out = '<div class="imagepress-main--single">
        <div>';
            $out .= '<div class="ip-bar">' .
                imagepress_editor() .
            '</div>';

            $out .= '<div class="imagepress-container">
                <a href="' . $post_thumbnail_url . '">' .
                    get_the_post_thumbnail( $image_id, $image_size_single ) .
                '</a>
            </div>';

            $out .= imagepress_get_images( $image_id, 0 );

            $out .= '<section role="navigation">' .
                get_previous_post_link( '%link', __( 'Previous', 'imagepress' ) ) . ' | ' .
                get_next_post_link( '%link', __( 'Next', 'imagepress' ) ) .
            '</section>
        </div>
        <div class="imagepress--image-sidebar">
            <p>' .
                get_avatar( $post->post_author, 80 ) . '<br>' .
                $imagepress_image_author_name;

                if ( (string) get_user_meta( $post->post_author, 'imagepress_role', true ) !== '' ) {
                    $out .= '<br><small>' . get_user_meta( $post->post_author, 'imagepress_role', true ) . '</small>';
                }

                $out .= '<br><small>' . __( 'Uploaded', 'imagepress' ) . ' <time title="' . get_the_time( get_option( 'date_format' ) ) . '">' . human_time_diff( get_the_time( 'U' ), current_time( 'timestamp' ) ) . ' ago</time> ' . __( 'in', 'imagepress' ) . ' ' . get_the_term_list( get_the_ID(), 'image-category', '', ', ', '' ) . '</small>
            </p>

            <div class="ip-bar">' .
                imagepress_get_post_like_link( $image_id ) . '<em> | </em>
                <i class="icofont-eye-alt"></i> ' . imagepress_get_post_views( $image_id ) .
                $ip_comments .
                '<em> | </em>';

                if ( function_exists( 'imagepress_frontend_add_collection' ) ) {
                    $out .= imagepress_frontend_add_collection( $image_id );
                }
            $out .= '</div>';

            if ( (int) get_option( 'image_sidebar_pattern_id' ) > 0 ) {
                $image_sidebar_pattern_id = get_post( get_option( 'image_sidebar_pattern_id' ) );

                $out .= '<div class="imagepress--image-sidebar-patern" data-id="' . get_option( 'image_sidebar_pattern_id' ) . '">' .
                    $image_sidebar_pattern_id->post_content .
                '</div>';
            }

            $out .= '<h1 class="ip-title">' .
                get_the_title( $image_id ) .
            '</h1>';

            $out .= wpautop( $post->post_content );

        $out .= '</div>
    </div>';

    return $out;
}




function imagepress_get_images( $post_id, $show ) {
    $thumbnail_id       = get_post_thumbnail_id();
    $images             = get_children(
        [
            'post_parent'    => $post_id,
            'post_status'    => 'inherit',
            'post_type'      => 'attachment',
            'post_mime_type' => 'image',
            'order'          => 'ASC',
            'orderby'        => 'menu_order ID',
        ]
    );
    $out                = '';
    $gallery_images     = '';
    $gallery_images_ids = [];

    if ( $images && count( $images ) > 0 ) {
        foreach ( $images as $attachment_id => $image ) {
            if ( (int) $image->ID !== (int) $thumbnail_id && (int) get_post_meta( $image->ID, 'imagepress_is_downloadable', true ) !== 1 ) {
                $big_array = image_downsize( $image->ID, 'full' );

                $gallery_images_ids[] = $image->ID;

                $gallery_images .= '<li class="blocks-gallery-item">
                    <figure>
                        <a href="' . get_permalink( $image->ID ) . '"><img src="' . $big_array[0] . '" alt="" data-id="' . $image->ID . '" data-link="' . get_permalink( $image->ID ) . '" class="wp-image-' . $image->ID . '"></a>
                    </figure>
                </li>';
            }
        }

        $out .= '<!-- wp:gallery {"ids":[' . implode( ',', $gallery_images_ids ) . '],"linkTo":"media"} -->
        <ul class="wp-block-gallery columns-3 is-cropped">' .
            $gallery_images .
        '</ul>';
    }

    $videos = get_children(
        [
            'post_parent' => $post_id,
            'post_status' => 'inherit',
            'post_type'   => 'attachment',
            'order'       => 'ASC',
            'orderby'     => 'menu_order ID',
        ]
    );

    if ( (int) $show === 1 ) {
        echo $out;
    } else {
        return $out;
    }
}

function imagepress_kformat( $number ) {
    $number = (int) $number;

    return number_format( $number, 0, '.', ',' );
}

function imagepress_related( $post_id ) {
    $author_id = get_post_field( 'post_author', $post_id );

    if ( (int) $author_id > 0 ) {
        $out  = '<h3>' . __( 'More by the same author', 'imagepress' ) . '</h3>';
        $out .= imagepress_get_related_author_posts( $author_id );

        return $out;
    }
}

function imagepress_author() {
    echo do_shortcode( '[cinnamon-profile]' );
}





function imagepress_return_image_sizes() {
    global $_wp_additional_image_sizes;

    $image_sizes = [];
    foreach ( get_intermediate_image_sizes() as $size ) {
        $image_sizes[ $size ] = [ 0, 0 ];
        if ( in_array( $size, [ 'thumbnail', 'medium', 'large' ] ) ) {
            $image_sizes[ $size ][0] = get_option( $size . '_size_w' );
            $image_sizes[ $size ][1] = get_option( $size . '_size_h' );
        } elseif ( isset( $_wp_additional_image_sizes ) && isset( $_wp_additional_image_sizes[ $size ] ) ) {
            $image_sizes[ $size ] = [ $_wp_additional_image_sizes[ $size ]['width'], $_wp_additional_image_sizes[ $size ]['height'] ];
        }
    }
    return $image_sizes;
}

function imagepress_order_list() {
    global $wpdb;

    $order = explode( ',', $_POST['order'] );

    foreach ( $order as $key => $value ) {
        $wpdb->query( $wpdb->prepare( "UPDATE `" . $wpdb->prefix . "posts` SET `menu_order` = %d WHERE `ID` = %d", $key, $value ) );
    }
}
add_action( 'wp_ajax_imagepress_list_update_order','imagepress_order_list' );
add_action( 'wp_ajax_nopriv_imagepress_list_update_order','imagepress_order_list' );



/*
 * Refactoring of option management functions.
 * Use a get_option() wrapper.
 */
function imagepress_get_option( $option ) {
    $ipOptions = get_option( 'imagepress' );

    return $ipOptions[ $option ];
}

function imagepress_update_option($optionArray) {
    $imagePressOption = get_option('imagepress');
    $updatedArray = array_merge($imagePressOption, $optionArray);

    update_option('imagepress', $updatedArray);
}



function imagepress_get_profile_uri( $author_id, $structure = true ) {
    $ip_profile_page_id  = (int) imagepress_get_option( 'ip_profile_page' );
    $ip_profile_page_url = get_permalink( $ip_profile_page_id );
    $ip_profile_slug     = (string) imagepress_get_option( 'cinnamon_author_slug' );

    $ip_profile_link = '<span class="name">
        ' . get_avatar( $author_id, 16 ) . ' <a href="' . $ip_profile_page_url . '?' . $ip_profile_slug . '=' . get_the_author_meta( 'user_nicename', $author_id ) . '"><b>' . get_the_author_meta( 'nickname', $author_id ) . '</b></a>
    </span>';

    if ( $structure === false ) {
        $ip_profile_link = $ip_profile_page_url . '?' . $ip_profile_slug . '=' . get_the_author_meta( 'user_nicename', $author_id );
    }

    return $ip_profile_link;
}



function imagepress_render_grid_element( $element_id ) {
    $post_thumbnail_id = get_post_thumbnail_id( $element_id );

    $ip_click_behaviour    = imagepress_get_option( 'ip_click_behaviour' );
    $get_imagepress_title  = imagepress_get_option( 'ip_title_optional' );
    $get_imagepress_author = imagepress_get_option( 'ip_author_optional' );
    $get_imagepress_meta   = imagepress_get_option( 'ip_meta_optional' );
    $get_imagepress_views  = imagepress_get_option( 'ip_views_optional' );
    $get_imagepress_likes  = imagepress_get_option( 'ip_likes_optional' );
    $get_ip_comments       = imagepress_get_option( 'ip_comments' );

    $size = imagepress_get_option( 'ip_image_size' );

    if ( $ip_click_behaviour === 'media' ) {
        // Get attachment source
        $image_attributes = wp_get_attachment_image_src( $post_thumbnail_id, 'full' );

        $ip_image_link = $image_attributes[0];
    } elseif ( $ip_click_behaviour === 'custom' ) {
        $ip_image_link = get_permalink( $element_id );
    }

    // Make all "brick" elements optional and active by default
    $ip_title_optional = '';
    if ( (int) $get_imagepress_title === 1 ) {
        $ip_title_optional = '<span class="imagetitle">' . get_the_title( $element_id ) . '</span>';
    }

    $ip_author_optional = '';
    if ( (int) $get_imagepress_author === 1 ) {
        // Get post author ID
        $post_author_id = get_post_field( 'post_author', $element_id );

        if ( (int) $post_author_id === 0 ) {
            $ip_author_optional = '<span class="name">' . __( 'Anonymous', 'imagepress' ) . '</span>';
        } else {
            $ip_author_optional = imagepress_get_profile_uri( $post_author_id );
        }
    }

    $ip_meta_optional = '';
    if ( (int) $get_imagepress_meta === 1 ) {
        $ip_meta_optional = '<span class="imagecategory ui--d" data-tag="' . strip_tags( get_the_term_list( $element_id, 'image-category', '', ', ', '' ) ) . '">' . strip_tags( get_the_term_list( $element_id, 'image-category', '', ', ', '' ) ) . '</span>';
    }

    $ip_views_optional = '';
    if ( (int) $get_imagepress_views === 1 ) {
        $ip_views_optional = '<span class="imageviews"><i class="icofont-eye-alt"></i> ' . imagepress_get_post_views( $element_id ) . '</span> ';
    }

    $ip_comments = '';
    if ( (int) $get_ip_comments === 1 ) {
        $ip_comments = '<span class="imagecomments"><i class="icofont-speech-comments"></i> ' . get_comments_number( $element_id ) . '</span> ';
    }

    $ip_likes_optional = '';
    if ( (int) $get_imagepress_likes === 1 ) {
        $ip_likes_optional = '<span class="imagelikes"><i class="icofont-heart"></i> ' . imagepress_get_like_count( $element_id ) . '</span> ';
    }

    $image_attributes = wp_get_attachment_image_src( $post_thumbnail_id, $size );
    $image_height     = (int) get_option( 'ip_box_height' ) > 0 ? (int) get_option( 'ip_box_height' ) : 250;

    $out = '<div class="ip_box ip_box_' . $element_id . '">
        <div class="ip-box--inner">
            <a href="' . $ip_image_link . '" data-taxonomy="' . strip_tags( get_the_term_list( $element_id, 'image-category', '', ', ', '' ) ) . '" data-src="' . $image_attributes[0] . '" title="' . get_the_title( $element_id ) . '" class="ip-anchor">
                <img class="ip-box-img" src="' . $image_attributes[0] . '" width="' . $image_attributes[1] . '" height="' . $image_attributes[2] . '" alt="' . get_the_title( $element_id ) . '" loading="lazy" style="height:' . $image_height . 'px">
            </a>
            <div class="ip_box_top">' . $ip_title_optional . $ip_author_optional . $ip_meta_optional . '</div>
            <div class="ip-box-bottom">' . $ip_views_optional . $ip_comments . $ip_likes_optional . '</div>
        </div>
    </div>';

    return $out;
}



/**
 * Display all public collections
 *
 * @return string
 */
function imagepress_collection_dropdown() {
    global $wpdb;

    $result = $wpdb->get_results( "SELECT * FROM " . $wpdb->prefix . "ip_collections WHERE collection_status = 1", ARRAY_A );

    $out = '<hr>
    <p>
        <select name="ip_collections" id="ip-collections">
            <option value="">' . __( 'Choose a collection...', 'imagepress' ) . '</option>';
            foreach ( $result as $collection ) {
                $out .= '<option value="' . $collection['collection_ID'] . '">' . $collection['collection_title'] . '</option>';
            }
        $out .= '</select>
    </p>' .
    __( 'or', 'imagepress' ) .
    '<p>
        <input type="text" name="ip_collections_new" id="ip-collections-new" placeholder="' . __( 'Create new collection...', 'imagepress' ) . '">
        <select name="collection_status" id="collection_status">
            <option value="1">' . __( 'Public', 'imagepress' ) . '</option>
            <option value="0">' . __( 'Private', 'imagepress' ) . '</option>
        </select>
    </p>
    <hr>';

    return $out;
}

/**
 * Upload secondary images
 */
function imagepress_upload_secondary($filesArray, $post_id) {
    if ((int) imagepress_get_option('ip_upload_secondary') === 1) {
        if ($filesArray) {
            require_once ABSPATH . "wp-admin" . '/includes/image.php';
            require_once ABSPATH . "wp-admin" . '/includes/file.php';
            require_once ABSPATH . "wp-admin" . '/includes/media.php';

            foreach ($filesArray['name'] as $key => $value) {
                if ($filesArray['name'][$key]) {
                    $file = [
                        'name' => $filesArray['name'][$key],
                        'type' => $filesArray['type'][$key],
                        'tmp_name' => $filesArray['tmp_name'][$key],
                        'error' => $filesArray['error'][$key],
                        'size' => $filesArray['size'][$key]
                    ];
                }
                $_FILES = ["attachment" => $file];
                foreach ($_FILES as $file => $array) {
                    $attach_id = media_handle_upload($file, $post_id);
                }
            }
        }
    }
}

/**
 * Upload downloadable image
 */
function imagepress_upload_downloadable( $files_array, $image_id ) {
    if ( $files_array ) {
        require_once ABSPATH . 'wp-admin/includes/image.php';
        require_once ABSPATH . 'wp-admin/includes/file.php';
        require_once ABSPATH . 'wp-admin/includes/media.php';

        foreach ( $files_array['name'] as $key => $value ) {
            if ( $files_array['name'][ $key ] ) {
                $file = [
                    'name'     => $files_array['name'][ $key ],
                    'type'     => $files_array['type'][ $key ],
                    'tmp_name' => $files_array['tmp_name'][ $key ],
                    'error'    => $files_array['error'][ $key ],
                    'size'     => $files_array['size'][ $key ],
                ];
            }

            $_FILES = [ 'attachment' => $file ];

            foreach ( $_FILES as $file => $array ) {
                $attach_id = media_handle_upload( $file, $image_id );

                update_post_meta( $attach_id, 'imagepress_is_downloadable', 1 );
            }
        }
    }
}
















/**
 * Log into WordPress
 *
 * Logs the user into WordPress, registers a new account or recovers the password
 *
 * @return string $out
 */
function imagepress_login() {
    global $user_ID, $user_identity;

    $out = '';

    if ( ! $user_ID ) {
        $register = ( isset( $_GET['register'] ) ) ? sanitize_text_field( $_GET['register'] ) : false;
        $reset    = ( isset( $_GET['reset'] ) ) ? sanitize_text_field( $_GET['reset'] ) : false;

        if ( (bool) $register === true ) {
            $out .= '<h3>' . __( 'Success!', 'imagepress' ) . '</h3>
            <p>' . __( 'Check your email for the password and then return to log in.', 'imagepress' ) . '</p>';
        } elseif ( (bool) $reset === true ) {
            $out .= '<h3>' . __( 'Success!', 'imagepress' ) . '</h3>
            <p>' . __( 'Check your email to reset your password.', 'imagepress' ) . '</p>';
        } else {
            $out .= '<h3>' . __( 'Have an account?', 'imagepress' ) . '</h3>
            <p>' . __( 'Log in or sign up!', 'imagepress' ) . '</p>';
        }

        $out .= '<div class="wp-block-columns" style="gap:2em">
            <div class="wp-block-column">
                <h3>' . __( 'Login', 'imagepress' ) . '</h3>

                <form method="post" action="' . wp_login_url() . '">
                    <p>
                        <label for="user_login">' . __( 'Username', 'imagepress' ) . '</label><br>
                        <input type="text" name="log" value="" size="20" id="user_login">
                    </p>
                    <p>
                        <label for="user_pass">' . __( 'Password', 'imagepress' ) . '</label><br>
                        <input type="password" name="pwd" value="" size="20" id="user_pass">
                    </p>

                    <div class="login_fields">
                        <label for="rememberme">
                            <input type="checkbox" name="rememberme" value="forever" id="rememberme" checked> ' . __( 'Remember me', 'imagepress' ) . '
                        </label>

                        <input type="submit" name="user-submit" value="' . __( 'Login', 'imagepress' ) . '">
                        <input type="hidden" name="redirect_to" value="' . $_SERVER['REQUEST_URI'] . '">
                        <input type="hidden" name="user-cookie" value="1">
                    </div>
                </form>
            </div>
            <div class="wp-block-column">
                <h3>' . __( 'Register', 'imagepress' ) . '</h3>

                <form method="post" action="' . wp_registration_url() . '">
                    <p>
                        <label for="user_login_signup">' . __( 'Username', 'imagepress' ) . '</label><br>
                        <input type="text" name="user_login" value="" size="20" id="user_login_signup">
                    </p>
                    <p>
                        <label for="user_email">' . __( 'Email Address', 'imagepress' ) . '</label><br>
                        <input type="text" name="user_email" value="" size="25" id="user_email">
                    </p>

                    <div class="login_fields">';

                        $out     .= '<input type="submit" name="user-submit" value="' . __( 'Sign Up', 'imagepress' ) . '">';
                        $register = ( isset( $_GET['register'] ) ) ? sanitize_text_field( $_GET['register'] ) : false;

                        if ( (bool) $register === true ) {
                            $out .= '<p>' . __( 'Check your email for the password!', 'imagepress' ) . '</p>';
                        }

                        $out .= '<input type="hidden" name="redirect_to" value="' . $_SERVER['REQUEST_URI'] . '?register=true">
                        <input type="hidden" name="user-cookie" value="1">
                    </div>
                </form>
            </div>
            <div class="wp-block-column">
                <h3>' . __( 'Forgot your password?', 'imagepress' ) . '</h3>

                <p>Enter your username or email to reset your password.</p>
                <form method="post" action="' . site_url( 'wp-login.php?action=lostpassword', 'login_post' ) . '">
                    <p>
                        <label for="user_login" class="hide">' . __( 'Username or Email Address', 'imagepress' ) . '</label>
                        <input type="text" name="user_login" value="" size="20" id="user_login">
                    </p>

                    <div class="login_fields">';

                        $out  .= '<input type="submit" name="user-submit" value="' . __( 'Reset Password', 'imagepress' ) . '">';
                        $reset = ( isset( $_GET['reset'] ) ) ? sanitize_text_field( $_GET['reset'] ) : false;

                        if ( (bool) $reset === true ) {
                            echo '<p>' . __( 'An email message will be sent to your email address.', 'imagepress' ) . '</p>';
                        }

                        $out .= '<input type="hidden" name="redirect_to" value="' . $_SERVER['REQUEST_URI'] . '?reset=true">
                        <input type="hidden" name="user-cookie" value="1">
                    </div>
                </form>
            </div>
        </div>';
    } else {
        // $user_identity = $current_user->display_name;
        $out .= '<h3>' . __( 'Welcome', 'imagepress' ) . ', ' . imagepress_get_profile_uri( get_current_user_id() ) . '</h3>
        <div class="userinfo"></div>

        <p>
            <a href="' . get_permalink( imagepress_get_option( 'ip_profile_page' ) ) . '">' . __( 'View Profile', 'imagepress' ) . '</a> | 
            <a href="' . get_permalink( imagepress_get_option( 'cinnamon_edit_page' ) ) . '">' . __( 'Edit Profile', 'imagepress' ) . '</a>
        </p>';
    }

    return $out;
}

add_shortcode( 'cinnamon-login', 'imagepress_login' );
add_shortcode( 'imagepress-login', 'imagepress_login' );



/**
 * Get the ID of the attachment with an 'imagepress_is_downloadable' meta key with a value of 1.
 *
 * @param mixed $parent_id Parent image custom post type ID.
 * @return void
 */
function imagepress_get_downloadable_attachment_id( $parent_id ) {
    global $wpdb;

    $query = $wpdb->prepare(
        "SELECT p.ID
        FROM {$wpdb->posts} p
        LEFT JOIN {$wpdb->postmeta} pm ON p.ID = pm.post_id
        WHERE p.post_type = 'attachment'
        AND p.post_parent = %d
        AND pm.meta_key = 'imagepress_is_downloadable'
        AND pm.meta_value = 1",
        $parent_id
    );

    $attachment_id = $wpdb->get_var( $query );

    return $attachment_id;
}



function imagepress_create_content( $type, $title, $content, $author, $parent_id = null ) {
    $query = new WP_Query(
        [
            'post_type'      => $type,
            'post_status'    => 'publish',
            'posts_per_page' => 1,
            'fields'         => 'ids',
            'title'          => $title,
        ]
    );

    if ( $query->have_posts() ) {
        $page_id = $query->posts[0];

        echo ucwords( $type ) . ' already exists: ' . $title . '<br>';

        return $page_id;
    }

    $page_id = wp_insert_post(
        [
            'comment_status' => 'close',
            'ping_status'    => 'close',
            'post_author'    => 1,
            'post_title'     => sanitize_text_field( $title ),
            'post_name'      => sanitize_title( $title ),
            'post_status'    => 'publish',
            'post_content'   => $content,
            'post_type'      => $type,
            'post_parent'    => $parent_id,
        ]
    );

    if ( (string) $type === 'wp_block' ) {
        // Make the new post a reusable block.
        if ( 0 !== $page_id && ! is_wp_error( $page_id ) ) {
            $saved = wp_set_post_terms( $page_id, 'wp_block', 'post_type' );
        }
    }

    echo '<p>Created ' . $type . ' <code>' . $page_id . '</code> for <b>' . $title . '</b></p>';

    return $page_id;
}



/**
 * Delete attached images when deleting a custom post type "image".
 *
 * @param int $post_id The ID of the post being deleted.
 */
function imagepress_delete_attached_images_on_post_delete( $post_id ) {
    if ( get_post_type( $post_id ) === imagepress_get_option( 'ip_slug' ) && (int) get_option( 'imagepress_delete_with_parent' ) === 1 ) {
        // Get all attached images
        $attachments = get_attached_media( '', $post_id );

        // Loop through the attachments and delete them
        foreach ( $attachments as $attachment ) {
            wp_delete_attachment( $attachment->ID, true ); // Set the second parameter to true to permanently delete the media file
        }
    }
}

// Hook the function to the before_delete_post action
add_action( 'before_delete_post', 'imagepress_delete_attached_images_on_post_delete' );
