<?php
/**
 * Add meta box
 *
 * @param post $post The post object
 */
function imagepress_add_meta_boxes( $post ) {
    $imagepress_slug = sanitize_text_field( imagepress_get_option( 'ip_slug' ) );

    add_meta_box( 'imagepress_meta_box', __( 'Image Settings', 'imagepress' ), 'imagepress_build_meta_box', [ $imagepress_slug ], 'normal', 'core' );
}



/**
 * Build custom field meta box
 *
 * @param post $post The post object
 */
function imagepress_build_meta_box( $post ) {
    wp_nonce_field( basename( __FILE__ ), 'imagepress_meta_box_nonce' );

    $imagepress_image_author_name  = get_post_meta( $post->ID, 'imagepress_image_author_name', true );
    $imagepress_image_author_email = get_post_meta( $post->ID, 'imagepress_image_author_email', true );

    $out = '';

    // Guest uploads require a name and an email
    if ( (int) imagepress_get_option( 'ip_registration' ) === 1 ) {
        $out .= '<p>
            <label>' . __( 'Name', 'imagepress' ) . '</label>
            <input type="text" id="imagepress_image_author_name" name="imagepress_image_author_name" placeholder="' . __( 'Name', 'imagepress' ) . '" value="' . $imagepress_image_author_name . '">
        </p>';
        $out .= '<p>
            <label>' . __( 'Email', 'imagepress' ) . '</label>
            <input type="email" id="imagepress_image_author_email" name="imagepress_image_author_email" placeholder="' . __( 'Email', 'imagepress' ) . '" value="' . $imagepress_image_author_email . '">
        </p>';
    }
    //

    echo $out;
}

/**
 * Store custom field meta box data
 *
 * @param int $post_id The post ID.
 */
function imagepress_save_meta_box_data( $post_id ) {
    if ( ! isset( $_POST['imagepress_meta_box_nonce'] ) || ! wp_verify_nonce( $_POST['imagepress_meta_box_nonce'], basename( __FILE__ ) ) ) {
        return;
    }
    if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) {
        return;
    }
    if ( ! current_user_can( 'edit_post', $post_id ) ) {
        return;
    }

    $imagepress_image_author_name  = isset( $_POST['imagepress_image_author_name'] ) ? sanitize_text_field( $_POST['imagepress_image_author_name'] ) : '';
    $imagepress_image_author_email = isset( $_POST['imagepress_image_author_email'] ) ? sanitize_text_field( $_POST['imagepress_image_author_email'] ) : '';

    update_post_meta( $post_id, 'imagepress_image_author_name', $imagepress_image_author_name );
    update_post_meta( $post_id, 'imagepress_image_author_email', $imagepress_image_author_email );
}

add_action( 'add_meta_boxes', 'imagepress_add_meta_boxes' );

add_action( 'save_post', 'imagepress_save_meta_box_data' );
