<?php
if ( isset( $_POST['imagepress_setup_save'] ) ) {
    if ( (int) $_POST['imagepress_preset_pages'] === 1 ) {
        /**
         * Generate default settings and options
         */
        $ip_updated_options = [
            'ip_click_behaviour'   => 'custom',
            'ip_box_ui'            => 'overlay',
            'ip_slug'              => 'image',
            'cinnamon_author_slug' => 'author',
            'ip_image_size'        => 'imagepress_sq_std',
            'ip_title_optional'    => 1,
            'ip_meta_optional'     => 1,
            'ip_views_optional'    => 1,
            'ip_likes_optional'    => 1,
            'ip_comments'          => 1,
            'ip_author_optional'   => 1,
            'ip_ipp'               => 24,
            'ip_order'             => 'DESC',
            'ip_orderby'           => 'date',
            'ip_app'               => 24,
            'ip_cards_per_author'  => 5,
        ];

        $ip_options = get_option( 'imagepress' );
        $ip_update  = array_merge( $ip_options, $ip_updated_options );

        update_option( 'imagepress', $ip_update );

        update_option( 'imagepress_use_template', 1 );
        update_option( 'ip_box_height', 280 );
        update_option( 'ip_box_row', 3 );

        update_option( 'default_role', 'author' );

        update_option( 'image_size_single', 'large' );

        update_option( 'imagepress_grid_width', 'default' );
        update_option( 'cinnamon_card_width', 100 );
        update_option( 'imagepress_form_description', 1 );

        /**
         * Create required gallery pages
         */
        $content = '<!-- wp:shortcode -->
    [imagepress-loop]
    <!-- /wp:shortcode -->';

        $gallery_page_id = imagepress_create_content( 'page', 'Gallery', $content, (int) $_POST['imagepress_preset_author'] );

        /**
         * Create required upload pages
         */
        $content = '<!-- wp:shortcode -->
    [imagepress-add]
    <!-- /wp:shortcode -->';

        $upload_page_id = imagepress_create_content( 'page', 'Upload', $content, (int) $_POST['imagepress_preset_author'] );

        /**
         * Create required author pages
         */
        $content = '<!-- wp:shortcode -->
    [cinnamon-profile]
    <!-- /wp:shortcode -->';

        $profile_page_id = imagepress_create_content( 'page', 'Profile', $content, (int) $_POST['imagepress_preset_author'] );

        $content = '<!-- wp:shortcode -->
    [cinnamon-login]
    <!-- /wp:shortcode -->';

        $login_page_id = imagepress_create_content( 'page', 'Login', $content, (int) $_POST['imagepress_preset_author'] );

        $content = '<!-- wp:shortcode -->
    [cinnamon-profile-edit]
    <!-- /wp:shortcode -->';

        $account_page_id = imagepress_create_content( 'page', 'My Account', $content, (int) $_POST['imagepress_preset_author'] );
        update_option( 'cinnamon_redirect_page', $account_page_id );

        /**
         * Create required Terms & Conditions pages
         */
        $terms_page_id = imagepress_create_content( 'page', 'Terms & Conditions', $content, (int) $_POST['imagepress_preset_author'] );

        // Update all options
        $ip_updated_options = [
            'ip_profile_page'       => $profile_page_id,
            'cinnamon_account_page' => get_permalink( $login_page_id ),
            'cinnamon_edit_page'    => $account_page_id,
            'ip_upload_tos'         => $terms_page_id,
        ];

        $ip_options = get_option( 'imagepress' );
        $ip_update  = array_merge( $ip_options, $ip_updated_options );

        update_option( 'imagepress', $ip_update );
    }

    if ( (int) $_POST['imagepress_preset_categories'] === 1 ) {
        /**
         * Create 3 random categories
         */
        $default_categories = [
            'Photography',
            'Concept Art',
            'Paintings',
        ];

        foreach ( $default_categories as $cat ) {
            if ( ! term_exists( $cat, 'image-category' ) ) {
                wp_insert_term( $cat, 'image-category' );
            }
        }
    }

    $ip_updated_options = [
        'ip_moderate'     => $_POST['ip_moderate'],
        'ip_registration' => $_POST['ip_registration'],
    ];

    $ip_options = get_option( 'imagepress' );
    $ip_update  = array_merge( $ip_options, $ip_updated_options );

    update_option( 'imagepress', $ip_update );

    //
    flush_rewrite_rules();
}
?>

<h2>ImagePress Setup</h2>

<p>This section will help you configure ImagePress by creating the required pages and setting up essential features.</p>
<p>You can skip this step and configure ImagePress manually. Note that more advanced options are available in the other sections.</p>
<p>If your single image template is not visible or you changed the slug recently, visit your <b>Permalinks</b> section and resave the changes.</p>

<form method="post" action="">
    <table class="form-table">
        <tbody>
            <!--
            <tr>
                <th scope="row"><label>Custom Presets</label></th>
                <td>
                    <p>
                        <select name="supernova_preset_custom" id="supernova_preset_custom">
                            <option value="0">Select a preset...</option>
                            <option value="1">Enterprise Blue #1 (CB)</option>
                            <option value="76">Basic Blue</option>
                        </select>
                    </p>
                </td>
            </tr>
            -->
            <tr>
                <th scope="row"></th>
                <td><h3 class="imagepress-onboarding-step"><img src="<?php echo trailingslashit( IMAGEPRESS_PLUGIN_URL ); ?>img/onboarding-star.svg" height="24" alt=""> Current Status</h3></td>
            </tr>
            <tr>
                <th scope="row"></th>
                <td>
                    <ul>
                        <?php
                        $author_slug      = imagepress_get_option( 'cinnamon_author_slug' );
                        $author_login_url = imagepress_get_option( 'cinnamon_account_page' );
                        $author_edit_url  = imagepress_get_option( 'cinnamon_edit_page' );
                        $ip_profile_page  = imagepress_get_option( 'ip_profile_page' );

                        $single_template = 'single-' . $ip_slug . '.php';

                        if ( '' !== locate_template( $single_template ) ) {
                            echo '<li><span class="dashicons dashicons-yes"></span> You have a custom image template available.</li>';
                        }

                        if ( empty( $author_slug ) ) {
                            echo '<li><span class="dashicons dashicons-no"></span> Your author slug is not set. Go to <b>Authors</b> section and set it.</li>';
                        } else {
                            echo '<li><span class="dashicons dashicons-yes"></span> Your author slug is <code>' . sanitize_text_field( $author_slug ) . '</code>. If you changed it recently, visit your <b>Permalinks</b> section and resave the changes.</li>';
                        }
                        if ( (int) $ip_profile_page === 0 ) {
                            echo '<li><span class="dashicons dashicons-no"></span> Your profile page is not set. Go to <b>Authors</b> section and set it.</li>';
                        } else {
                            echo '<li><span class="dashicons dashicons-yes"></span> Your profile page is <b>' . get_the_title( $ip_profile_page ) . '</b>. If you changed it recently, visit your <b>Permalinks</b> section and resave the changes.</li>';
                        }
                        if ( empty( $author_edit_url ) ) {
                            echo '<li><span class="dashicons dashicons-no"></span> Your author profile edit URL is not set. Go to <b>Authors</b> section and set it.</li>';
                        } else {
                            echo '<li><span class="dashicons dashicons-yes"></span> Your author profile edit URL is <code>' . esc_url( get_permalink( $author_edit_url ) ) . '</code>.</li>';
                        }
                        if ( (string) get_option( 'default_role' ) === 'author' ) {
                            echo '<li><span class="dashicons dashicons-yes"></span> New user default role is <code>author</code>. Subscribers and contributors are not able to edit their uploaded images.</li>';
                        } else {
                            echo '<li><span class="dashicons dashicons-no"></span> New user default role should be <code>author</code> in order to allow for front-end image editing. Subscribers and contributors are not able to edit their uploaded images.</li>';
                        }
                        ?>
                    </ul>
                </td>
            </tr>
            <tr>
                <th scope="row"></th>
                <td><h3 class="imagepress-onboarding-step"><img src="<?php echo trailingslashit( IMAGEPRESS_PLUGIN_URL ); ?>img/onboarding-star.svg" height="24" alt=""> Step 1</h3></td>
            </tr>
            <tr>
                <th scope="row"><label>Content</label></th>
                <td>
                    <p>
                        <input type="checkbox" id="imagepress-preset-pages" name="imagepress_preset_pages" value="1">
                        <label for="imagepress-preset-pages">Create default pages</label>
                        <br><small>Create <b>Gallery</b>, <b>Upload</b>, <b>Profile</b>, <b>Account Settings</b> and <b>Terms & Conditions</b> pages.</small>
                    </p>
                    <p>
                        <input type="checkbox" id="imagepress-preset-categories" name="imagepress_preset_categories" value="1">
                        <label for="imagepress-preset-categories">Create example categories</label>
                        <br><small>Create 3 example categories. Can be edited or removed later.</small>
                    </p>
                    <p>
                        <label>Assign content (pages) to user:</label>
                        <?php
                        $user = get_user_by( 'id', get_option( 'imagepress_preset_author' ) );

                        if ( $user != null ) {
                            wp_dropdown_users(
                                [
                                    'id'       => 'imagepress_preset_author',
                                    'name'     => 'imagepress_preset_author',
                                    'role'     => 'administrator',
                                    'selected' => get_option( 'imagepress_preset_author' ),
                                ]
                            );
                        } else {
                            wp_dropdown_users(
                                [
                                    'id'   => 'imagepress_preset_author',
                                    'name' => 'imagepress_preset_author',
                                    'role' => 'administrator',
                                ]
                            );
                        }
                        ?>
                    </p>
                </td>
            </tr>
            <tr>
                <th scope="row"></th>
                <td><h3 class="imagepress-onboarding-step"><img src="<?php echo trailingslashit( IMAGEPRESS_PLUGIN_URL ); ?>img/onboarding-star.svg" height="24" alt=""> Step 2</h3></td>
            </tr>
            <tr>
                <th scope="row"><label for="ip_registration">User Registration</label></th>
                <td>
                    <select name="ip_registration" id="ip_registration">
                        <option value="0"<?php selected( 0, (int) imagepress_get_option( 'ip_registration' ) ); ?>>Require user registration (recommended)</option>
                        <option value="1"<?php selected( 1, (int) imagepress_get_option( 'ip_registration' ) ); ?>>Do not require user registration</option>
                    </select>
                    <br><small>Require users to be registered and logged in to upload images (recommended).</small>
                </td>
            </tr>
            <tr>
                <th scope="row"></th>
                <td><h3 class="imagepress-onboarding-step"><img src="<?php echo trailingslashit( IMAGEPRESS_PLUGIN_URL ); ?>img/onboarding-star.svg" height="24" alt=""> Step 3</h3></td>
            </tr>
            <tr>
                <th scope="row"><label for="ip_moderate">Image Moderation</label></th>
                <td>
                    <select name="ip_moderate" id="ip_moderate">
                        <option value="0"<?php selected( 0, (int) imagepress_get_option( 'ip_moderate' ) ); ?>>Moderate all images (recommended)</option>
                        <option value="1"<?php selected( 1, (int) imagepress_get_option( 'ip_moderate' ) ); ?>>Do not moderate images</option>
                    </select>
                    <br><small>Moderate all submitted images (recommended).</small>
                </td>
            </tr>

        </tbody>
    </table>

    <hr>
    <p><input type="submit" name="imagepress_setup_save" value="Apply Changes" class="button button-primary"></p>
</form>
