<?php
add_action( 'wp_ajax_nopriv_imagepress-like', 'imagepress_like' );
add_action( 'wp_ajax_imagepress-like', 'imagepress_like' );

function imagepress_like() {
    if ( isset( $_POST['imagepress_like'] ) ) {
        $post_id      = (int) sanitize_text_field( $_POST['post_id'] );
        $ip_vote_meta = '_like_count';
        $like_count   = get_post_meta( $post_id, $ip_vote_meta, true ); // post like count

        if ( function_exists( 'wp_cache_post_change' ) ) { // invalidate WP Super Cache if exists
            $GLOBALS['super_cache_enabled'] = 1;

            wp_cache_post_change( $post_id );
        }

        if ( is_user_logged_in() ) {
            $user_id     = get_current_user_id(); // current user
            $meta_posts  = (array) get_user_option( '_liked_posts', $user_id ); // post ids from user meta
            $meta_users  = get_post_meta( $post_id, '_user_liked' ); // user ids from post meta
            $liked_posts = null; // setup array variable
            $liked_users = null; // setup array variable

            if ( count( $meta_posts ) !== 0 ) { // meta exists, set up values
                $liked_posts = $meta_posts;
            }

            if ( ! is_array( $liked_posts ) ) { // make array just in case
                $liked_posts = [];
            }

            if ( count( $meta_users ) !== 0 ) { // meta exists, set up values
                $liked_users = $meta_users[0];
            }

            if ( ! is_array( $liked_users ) ) { // make array just in case
                $liked_users = [];
            }

            $liked_posts[ 'post-' . $post_id ] = $post_id; // add post id to user meta array
            $liked_users[ 'user-' . $user_id ] = $user_id; // add user id to post meta array

            $user_likes = count( $liked_posts ); // count user likes

            if ( ! imagepress_already_liked( $post_id ) ) { // like the post
                update_post_meta( $post_id, '_user_liked', $liked_users ); // add user ID to post meta
                update_post_meta( $post_id, $ip_vote_meta, ++$like_count ); // +1 count post meta
                update_user_option( $user_id, '_liked_posts', $liked_posts ); // add post ID to user meta
                update_user_option( $user_id, '_user_like_count', $user_likes ); // +1 count user meta

                echo $like_count; // update count on front end

                // notification
                global $wpdb;
                $act_time = current_time( 'mysql', true );
                $wpdb->query( "INSERT INTO " . $wpdb->prefix . "notifications (ID, userID, postID, actionType, actionTime) VALUES (null, $user_id, $post_id, 'loved', '$act_time')" );
                //
            } else { // unlike the post
                $pid_key = array_search( $post_id, $liked_posts ); // find the key
                $uid_key = array_search( $user_id, $liked_users ); // find the key
                unset( $liked_posts[ $pid_key ] ); // remove from array
                unset( $liked_users[ $uid_key ] ); // remove from array
                $user_likes = count( $liked_posts ); // recount user likes
                update_post_meta( $post_id, '_user_liked', $liked_users ); // remove user ID from post meta
                update_post_meta( $post_id, $ip_vote_meta, --$like_count ); // -1 count post meta
                update_user_option( $user_id, '_liked_posts', $liked_posts ); // remove post ID from user meta
                update_user_option( $user_id, '_user_like_count', $user_likes ); // -1 count user meta

                echo 'already' . $like_count; // update count on front end
            }
        } else { // user is not logged in (anonymous)
            $user_ip   = $_SERVER['REMOTE_ADDR']; // user IP address
            $meta_ips  = get_post_meta( $post_id, '_user_IP' ); // stored IP addresses
            $liked_ips = null; // set up array variable

            if ( count( $meta_ips ) !== 0 ) { // meta exists, set up values
                $liked_ips = $meta_ips[0];
            }

            if ( ! is_array( $liked_ips ) ) { // make array just in case
                $liked_ips = [];
            }

            if ( ! in_array( $user_ip, $liked_ips ) ) { // if IP not in array
                $liked_ips[ 'ip-' . $user_ip ] = $user_ip; // add IP to array
            }

            if ( ! imagepress_already_liked( $post_id ) ) { // like the post
                update_post_meta( $post_id, '_user_IP', $liked_ips ); // add user IP to post meta
                update_post_meta( $post_id, $ip_vote_meta, ++$like_count ); // +1 count post meta

                echo $like_count; // update count on front end
            } else { // unlike the post
                $ip_key = array_search( $user_ip, $liked_ips ); // find the key
                unset( $liked_ips[ $ip_key ] ); // remove from array
                update_post_meta( $post_id, '_user_IP', $liked_ips ); // remove user IP from post meta
                update_post_meta( $post_id, $ip_vote_meta, --$like_count ); // -1 count post meta

                echo 'already' . $like_count; // update count on front end
            }
        }
    }

    exit;
}

/**
 * Test if user already liked post
 */
function imagepress_already_liked( $post_id ) {
    if ( is_user_logged_in() ) { // user is logged in
        $user_id     = get_current_user_id(); // current user
        $meta_users  = get_post_meta( $post_id, '_user_liked' ); // user ids from post meta
        $liked_users = ''; // set up array variable

        if ( count( $meta_users ) !== 0 ) { // meta exists, set up values
            $liked_users = $meta_users[0];
        }

        if ( ! is_array( $liked_users ) ) { // make array just in case
            $liked_users = [];
        }

        if ( in_array( $user_id, $liked_users ) ) { // true if user ID in array
            return true;
        }

        return false;
    } else { // user is anonymous, use IP address for voting
        $meta_ips   = get_post_meta( $post_id, '_user_IP' ); // get previously voted IP address
        $ip_address = $_SERVER['REMOTE_ADDR']; // retrieve current user IP
        $liked_ips  = ''; // set up array variable

        if ( count( $meta_ips ) !== 0 ) { // meta exists, set up values
            $liked_ips = $meta_ips[0];
        }

        if ( ! is_array( $liked_ips ) ) { // make array just in case
            $liked_ips = [];
        }

        if ( in_array( $ip_address, $liked_ips ) ) { // true if IP in array
            return true;
        }

        return false;
    }
}

/**
 * Front end button
 */
function imagepress_get_post_like_link( $post_id ) {
    $output         = '';
    $class          = '';
    $ip_vote_like   = get_option( 'ip_vote_like' ) !== '' ? get_option( 'ip_vote_like' ) : __( 'Like', 'imagepress' );
    $ip_vote_unlike = get_option( 'ip_vote_unlike' ) !== '' ? get_option( 'ip_vote_unlike' ) : __( 'Unlike', 'imagepress' );

    if ( is_user_logged_in() ) {
        $like = '<i class="icofont-heart icofont-heart--not-liked"></i><span class="ip-icon-label"> ' . $ip_vote_like . '</span>';

        if ( imagepress_already_liked( $post_id ) ) {
            $class = esc_attr( 'liked' );
            $like  = '<i class="icofont-heart"></i><span class="ip-icon-label"> ' . $ip_vote_unlike . '</span>';
        }

        $output = '<a href="#" class="imagepress-like ' . $class . '" data-post_id="' . $post_id . '">' . $like . '</a>';
    }

    return $output;
}

/**
 * If the user is logged in, output a list of posts that the user likes
 */
function imagepress_frontend_user_likes( $author ) {
    $like_list  = '<div class="cinnamon-likes" id="cinnamon-love">';
    $user_likes = get_user_option( '_liked_posts', $author );
    $the_likes  = '';

    if ( ! empty( $user_likes ) && count( $user_likes ) > 0 ) {
        $the_likes = $user_likes;
    }

    if ( ! is_array( $the_likes ) ) {
        $the_likes = [];
    }

    $the_likes = array_reverse( $the_likes );
    $count     = count( $the_likes );

    if ( $count > 0 ) {
        foreach ( $the_likes as $the_like ) {
            $like_list .= '<a href="' . esc_url( get_permalink( $the_like ) ) . '"><!--' . get_the_title( $the_like ) . '-->' . get_the_post_thumbnail( $the_like, 'thumbnail' ) . '</a>';
        }
    }
    $like_list .= '</div>';

    return $like_list;
}



function imagepress_get_like_users( $post_id ) {
    $meta_users = get_post_meta( $post_id, '_user_liked' );

    foreach ( $meta_users as $users ) {
        foreach ( $users as $user ) {
            echo '<a href="' . get_author_posts_url( $user ) . '" title="' . get_the_author_meta( 'nickname', $user ) . '">' . get_avatar( $user, 40 ) . '</a> ';
        }
    }
}

function imagepress_get_like_count( $post_id ) {
    $meta_users  = get_post_meta( $post_id, '_user_liked' );
    $total_users = array_sum( array_map( 'count', $meta_users ) );

    return $total_users;
}
